import java.util.LinkedList;

/**
 * The class represents an accelerometer.
 *
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Accelerometer extends ObjectNode {
    /** The lookup table for device specific mapping of the output values. */
    LinkedList<double[]> lookupTable = new LinkedList<double[]>();

    /**
     * The constructor.
     * 
     * @param name The accelermoters identifier name.
     * @param parent The reference to the parent object.
     */
    Accelerometer(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates the code for an accelerometer.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        code.indent();
        code.println("<Accelerometer name=\"" + name + "\">");
        
        // rotation fix
        rotation[0] = 0;
        generateCodeTransform(code);

        code.indent();
        code.println("<Element>");
        Box.generateCodeBox(code, name + "Box", 0.0001, 0.0001, 0.0001);
        code.println("</Element>");
        code.unindent();
        code.println("</Accelerometer>");
        code.unindent();
    }
}
