import java.lang.Math;

/**
 * The class represents a box.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Box extends Geometry {
    /** The size of the box in length, width and height. */
    double[] size;
    
    /** The counter to build a unique name */
    private static int count = 0;

    /**
     * The constructor.
     * 
     * @param name The box identifier name.
     * @param parent The reference to the parent object.
     */
    Box(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates a unique name.
     */
    public void generateName() {
        if(name != null)
            return;
        generateName("BOX" + count ++);
    }

    /**
     * The method generates the bounding code begin.
     * 
     * @param code The code output stream.
     * @param simple Is it a simple physical object?
     */
    public void generateCodeBoundingStart(TreeStream code, boolean simple) {
        code.indent();
        code.println("<" + (simple ? "Simple" : "") + "Box name=\"" + name + "\" length=\"" + size[0] + "\" width=\"" + size[1] + "\" height=\"" + size[2] + "\">");
    }

    /**
     * The method generates the bounding code closure.
     * 
     * @param code The code output stream.
     * @param simple Is it a simple physical object?
     */
    public void generateCodeBoundingEnd(TreeStream code, boolean simple) {
        code.println("</" + (simple ? "Simple" : "") + "Box>");
        code.unindent();
    }

    /**
     * The method generates a box with a specific name and size.
     * 
     * @param code The code output stream.
     * @param name The box identifier name.
     * @param lenght The length of the box.
     * @param width The width of the box.
     * @param height The height of the box.
     */
    static public void generateCodeBox(TreeStream code, String name, double length, double width, double height) {
        code.indent();
        code.println("<Box name=\"" + name + "_BOX\" length=\"" + length + "\" width=\"" + width + "\" height=\"" + height + "\"/>");
        code.unindent();
    }

    /**
     * The method generates the box code.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        generateCodeBoundingStart(code, false);
        generateCodeBoundingEnd(code, false);
        code.unindent();
    }
}
