/**
 * The class represents a camera.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Camera extends ObjectNode {
    /** The window position in the mainwindow where the camera picture is diplayed */
    double[] windowPosition;
    
    /** The zoom factor for the displayed camera picture */
    int pixelSize;
    
    /** The horizontal field of view angle of the camera in radiants */
    double fieldOfView;
    
    /** Width of the image in pixels */
    int width;
    
    /** Height of the image in pixels */
    int height;
    
    /** The distance to the near OpenGL clipping planes */
    double near;

    /**
     * The constructor.
     * 
     * @param name The camera identifier name.
     * @param parent The reference to the parent object.
     */
    Camera(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates the camera code.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        // Camera in Webots is rotated 180 degrees, not needed in SimRobot
        if(rotation != null)
            rotation[0] = 0f;

        Sphere.generateCodeSphereStart(code, false, name, 0.01);
        Physics.generateCodePhysicsDefault(code, 0.0001);

        code.indent();
        code.println("<Elements>");
        code.indent();
        code.println("<Camera name=\"" + name + "\">");

        generateCodeTransform(code);

        code.indent();
        code.println("<Resolution x=\"" + (width * 2) + "\" y=\"" + (height * 2) + "\" />");
        code.println("<Exposure time=\"0.0333\" />");
        code.println("<OpeningAngles x=\"45.08\" y=\"34.58\" />");
        code.println("<Range near=\"" + near + "\" far=\"10\" />");
        code.println("<PerspectiveProjection />");
        code.unindent();

        code.println("</Camera>");
        code.unindent();
        code.println("</Elements>");
        code.unindent();
        Sphere.generateCodeSphereEnd(code, false);
    }
}
