/**
 * The class represents the exception that are generated when a compilationerror occures.
 * The exception is caught and printed in the main method {@link Converter#main(String[]).
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class CompileException extends Exception {

    /**
     * The serialization unique identifier.
     */
    private static final long serialVersionUID = 1L;

    /**
     * The constructor.
     * @param message Die Fehlermeldung. Ihr wird der Text "Fehler in Zeile x,
     *         Spalte y: " vorangestellt, bzw. lediglich "Fehler: ", wenn die
     *         Quelltextstelle unbekannt ist.
     * @param position Die Quelltextstelle an der der Fehler aufgetreten ist.
     *         Dieser Parameter kann auch null sein, wenn die Stelle nicht
     *         zugeordnet werden kann.
     */
    CompileException(String message, Position position) {
        super("Error" + (position == null ? "" : " in row " + position.line +
                ", column " + position.column) + ": " + message);
    }
}
