import java.io.*;
/**
 * This is the programmclass to converte a Webots model to a SimRobot model.
 * 
 * <ul>
 *  <li>Webots reference manual: <a href="http://www.cyberbotics.com/cdrom/common/doc/webots/reference/reference.pdf">Link</a></li>
 *  <li>RoSiML: <a href="http://www.informatik.uni-bremen.de/spprobocup/RoSiML.html">Link</a></li>
 * </ul>
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 * 
 * @version 0.1
 */
public class Converter
{
    /** The syntax analysis object. */
    private SyntaxAnalyzer syntaxAnalyzer;
    
    /** The result of the syntax analysis. */
    private Robot syntaxTree;
    
    /** The stream object to format the outputcode. */
    private TreeStream codeExport;
    
    /** The simrobot object tree. */
    private SimRobotTree simRobotTree;

    /**
     * The constructor for objects of class Converter.
     */
    public Converter() {
    }

    /**
     * Starts the converion of the given model
     * 
     * @param: inputProto The input file with the proto model.
     * @param: outputRsi The output file with the generated simrobot model.
     * @param: printSymbols Enables printing of analysed symbols.
     * @param: roSi2 Create RiSi2 code instead of RoSi1
     */
    private void converte(String fileNameInput, String fileNameOutput, boolean printSymbols, boolean roSi2) {
        try
        {
            this.simRobotTree = new SimRobotTree();
            this.syntaxAnalyzer = new SyntaxAnalyzer(fileNameInput, printSymbols);
            this.syntaxTree = syntaxAnalyzer.parse();
            this.codeExport = new TreeStream((new FileOutputStream(fileNameOutput)), 2);
            if(roSi2)
              this.simRobotTree.generateRoSi2CodeRoot(codeExport);
            else
              this.simRobotTree.generateCode(codeExport);
        }
        catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }
    }

    /**
    * The entry point of the converter.
    * @param: args Arguments from the command line
    */
    public static void main(String[] args)  {
    
        String inputFile = "";
        String outputFile = "";
        boolean roSi2 = false;
    
        for(int i = 0; i < args.length; ++i) {
            if(args[i].equals("-2"))
                roSi2 = true;
            else if(inputFile.length() == 0)
                inputFile = args[i];
            else if(outputFile.length() == 0)
                outputFile = args[i];
            else {
                inputFile = "";
                break;
            }
        }
    
        if(inputFile.length() == 0) {
          System.err.println("Usage: Protoc.jar [-2] <input file> [<output file>]");
          return;
        }
        if(outputFile.length() == 0)
          outputFile = (inputFile.endsWith(".proto") ? inputFile.substring(0, inputFile.length() - 6) : inputFile) + (roSi2 ? ".rsi2" : ".rsi");
          
        System.out.println("input file: " + inputFile);
        System.out.println("output file: " + outputFile);
       
        new Converter().converte(inputFile, outputFile, false, roSi2);
    }

}
