import java.util.LinkedList;

/**
 * The class represents a distance sensor for example sonar, ultrasonic, infrared, laser, etc.. .
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class DistanceSensor extends ObjectNode {
    /** The lookup table representing the response curve of the sensor. */ 
    LinkedList<double[]> lookupTable = new LinkedList<double[]>(); //not used now
    
    /** The type of sensor ("infra-red","sonar","laser"). */
    String type;
    
    /** The number of rays to compute distance. */
    int numberOfRays;   //not used now
    
    /** The angle or radius for the sensor type. */
    double aperture;    //not used now

    /**
     * The constructor
     * 
     * @param name The name of the sensor.
     * @param parent The reference to the parent object.
     */
    DistanceSensor(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates the distance sensor.
     * 
     * @param code The code stream object
     */
    public void generateCode(TreeStream code) {
        code.indent();
        code.println("<SingleDistanceSensor name=\"" + name + "\">");
        generateCodeTransform(code);
        code.indent();
        code.println("<Range near=\"0.03\" far=\"0.7\" />");
        code.unindent();
        code.println("</SingleDistanceSensor>");
        code.unindent();
    }
}
