import java.util.LinkedList;

/**
 * The class represents a gyro.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Gyro extends ObjectNode {
    /** The lookup table for device specific mapping of the output values. */
    LinkedList<double[]> lookupTable = new LinkedList<double[]>();
    
    /** Disables the z-axis of the gyro. */
    boolean zAxis;

    /**
     * The constructor.
     * 
     * @param name The gyros identifier name.
     * @param parent The reference to the parent object.
     */
    Gyro(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates the code for a gyro.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        code.indent();
        code.println("<Gyroscope name=\"" + name + "\">");
        generateCodeTransform(code);
        code.indent();
        code.println("<Element>");
        Box.generateCodeBox(code, name + "Box", 0.0001, 0.0001, 0.0001);
        code.println("</Element>");
        code.unindent();
        code.println("</Gyroscope>");
        code.unindent();
    }
}
