/**
 * The class represents a led.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class LED extends ObjectNode {
    /** The led color. */
    double[] color;
    
    /** Defines the type of led in dependency of the colorlist. */
    boolean gradual;
    
    /** Defines the visual properties for a switched off led. */
    Appearance off;
    
    /** Defines the visual properties for a switched on led. */
    Appearance on;
    
    /** The counter to build an unique identifier. */
    private static int count = 0;

    /**
     * The constructor.
     * 
     * @param name The led identifier name.
     * @param parent The refernence to the parent object.
     */
    LED(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates an unique name.
     */
    public void generateName() {
        if(name != null)
            return;
        generateName("LED" + count ++);
    }

    /**
     * The method generates the on and off appearances for the led.
     */
    public void createAppearance() {
        for(Node child : children)
            if(child instanceof Shape) {
                off = ((Shape)child).appearance;
                break;
            }

        assert(off != null);

        if(color == null || color.length != 3) {
            on = off;
            return;
        }

        on = new Appearance(off.name + "_" + name + "_ON", this);
        on.material = (Material)off.material.clone();
        on.material.emissiveColor = color;
        on.texture = off.texture;
    }

    /**
     * The mehtod generates the led code.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        code.indent();
        code.println("<Led name=\"" + name + "\" appearanceWhenOn=\"" + on.name + "\" appearanceWhenOff=\"" + off.name + "\">");
        super.generateCode(code);
        code.println("</Led>");
        code.unindent();
    }
}
