/**
 * The class Matrix3D is just another implementation of a rotation matrix.
 * Only the methods that are actually used by other classes are implemented.
 * These methods were basically copied from SimRobot's class Matrix3d.
 *
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 */
class Matrix3D {
    /** The three columns of the matrix. */
    Vector3D col[] = {new Vector3D(), new Vector3D(), new Vector3D()};
    
    /** Default constructor. */
    Matrix3D() {}
    
    /**
     * Copy constructor.
     * @mat The matrix that is cloned. 
     */
    Matrix3D(Matrix3D mat)
    {
      col[0] = (Vector3D) mat.col[0].clone();
      col[1] = (Vector3D) mat.col[1].clone();
      col[2] = (Vector3D) mat.col[2].clone();
    }
    
    /**
     * Initialize the matrix as rotation around the x-axis.
     * @param angle The rotation around the x-axis in radians.
     */
    void setXRotation(double angle) {
        double c = Math.cos(angle);
        double s = Math.sin(angle);
        col[0].v[0] = 1.0; col[1].v[0] = 0.0; col[2].v[0] = 0.0;
        col[0].v[1] = 0.0; col[1].v[1] = c;   col[2].v[1] = -s;
        col[0].v[2] = 0.0; col[1].v[2] = s;   col[2].v[2] = c;
    }

    /**
     * Initialize the matrix as rotation around the y-axis.
     * @param angle The rotation around the y-axis in radians.
     */
    void setYRotation(double angle) {
        double c = Math.cos(angle);
        double s = Math.sin(angle);
        col[0].v[0] = c;   col[1].v[0] = 0.0; col[2].v[0] = s;
        col[0].v[1] = 0.0; col[1].v[1] = 1.0; col[2].v[1] = 0.0;
        col[0].v[2] = -s;   col[1].v[2] = 0.0; col[2].v[2] = c;
    }

    /**
     * Initialize the matrix as rotation around the z-axis.
     * @param angle The rotation around the z-axis in radians.
     */
    void setZRotation(double angle) {
        double c = Math.cos(angle);
        double s = Math.sin(angle);
        col[0].v[0] = c;   col[1].v[0] = -s;  col[2].v[0] = 0.0;
        col[0].v[1] = s;   col[1].v[1] = c;   col[2].v[1] = 0.0;
        col[0].v[2] = 0.0; col[1].v[2] = 0.0; col[2].v[2] = 1.0;
    }

    /**
     * Multiply this matrix with a vector.
     * @param vec The vector.
     * @return The result of the multiplication.
     */
    Vector3D mult(Vector3D vec) {
        return col[0].mult(vec.v[0]).add(col[1].mult(vec.v[1])).add(col[2].mult(vec.v[2]));
    }
}
