import java.util.LinkedList;

/**
 * The class represents the base node of all objects.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Node extends SimRobotTree implements Cloneable {
    /** The name of the defined object */
    String name;
    
    /** The reference to the parent object */
    Node parent;
    
    /** A static counter to make macronames unique */
    private static int count = 0;

    /**
     * The constructor.
     * 
     * @param name The name of the object
     * @param parent The reference to the parent object
     */
    Node(String name, Node parent) {
        this.name = name;
        this.parent = parent;
    }

    /**
     * The method generates a unique name for a node.
     */
    public void generateName() {
        if(name != null)
            return;
        for(Node p = parent; p != null; p = p.parent)
            if(p.name != null) {
                name = p.name + "_NODE" + count ++;
                break;
            }
        if(name == null)
            name = "NODE" + count ++;
    }
    
    /**
     * The method concatenates the addition to the name.
     * 
     * @param addition The addtion for the name.
     */
    public void generateName(String addition) {
        if(name != null)
            return;
        for(Node p = parent; p != null; p = p.parent)
            if(p.name != null) {
                name = p.name + "_" + addition;
                break;
            }
        if(name == null)
            name = addition;
    }

    /**
     * The methode makes the object clonable.
     */
    public Object clone() {
        Object theClone = null;
        try {
            theClone = super.clone();
        }
        catch(CloneNotSupportedException e) {
        }
        return theClone;
    }
    
    /**
     * The method is an abstract dummy. 
     * 
     * @param code The code stream object.
     */
    void generateCode(TreeStream code) {
    }
}
