/**
 * The class represents a node with physical attributes.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */

class Physics extends Node {
    /** The density of an object */
    double density;
    
    /** The mass of an object */
    double mass;
    
    /** The center of mass of an object */
    double[] centerOfMass;

    /**
     * The constructor.
     * 
     * @param name The objects identifier name.
     * @param parent The reference to the parent object.
     */
    Physics(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates default physical attributes needed by RoSiML.
     * 
     * @param code The code output stream.
     * @param mass The mass attribute.
     */
    static public void generateCodePhysicsDefault(TreeStream code, double mass) {
        code.indent();
        code.println("<PhysicalAttributes>");
        code.indent();
        code.println("<Mass value=\"" + mass + "\" />");
        code.unindent();
        code.println("</PhysicalAttributes>");
        code.unindent();
    }

    /**
     * The method generates empty physical attributes needed by RoSiML.
     * 
     * @param code The code output stream.
     */
    static public void generateCodePhysicsEmpty(TreeStream code) {
        code.indent();
        code.println("<PhysicalAttributes />");
        code.unindent();
    }

    /**
     * The method generates the pysical attributes code with all attributes.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        code.indent();
        code.println("<PhysicalAttributes>");
        code.indent();
        if(mass != 0)
          code.println("<Mass value=\"" + mass + "\" />");
        if(centerOfMass != null)
          code.println("<CenterOfMass x=\"" + centerOfMass[0] + "\" y=\"" + centerOfMass[1] + "\" z=\"" + centerOfMass[2] + "\" />");
        code.unindent();
        code.println("</PhysicalAttributes>");
        code.unindent();
    }
}
