/**
 * The class represents a position in source code.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Position {
    /** The row in the source code. */
    int line;

    /** The column in the source code. */
    int column;

    /**
     * The constructor.
     * @param line The row in the source code.
     * @param column The column in the source code.
     */
    Position(int line, int column) {
        this.line = line;
        this.column = column;
    }

    /**
     * The method increments the count of the source code position.
     * Therefore the method inspects the actual character.
     * @param c The actual read character. '\n' jumps to the beginning of the next row
     * '\t' jumps to the next tab (a multiple of 8)
     * '\r' is ignored
     */
    void next(char c) {
        if (c == '\n') {
            ++line;
            column = 1;
        } else if (c == '\t') {
            column += 8 - (column - 1) % 8;
        } else if (c != '\r') {
            ++column;
        }
    }
}
