import java.util.LinkedList;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * The class represents a robot object
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Robot extends ObjectNode
{
    /**
     * The constructor
     * 
     * @param name The robots identifier name.
     * @param parent Reference to the parent object.
     */
    Robot(String name, Node parent) {
        super(name, parent);
        this.robot.add(this);
    }

    /**
     * Generates the RoSimL output code 
     *
     * @param code The code ouptut stream.
     */
    public void generateCode(TreeStream code) {
        //generates the definition of the blue robot
        String name2 = "Nao";
        code.indent();
        code.println("<Macro name=\"" + name2 + "-blue\">");
        code.indent();
        code.println("<Movable name=\"" + name2 + "\">");
        super.generateCode(code);
        code.println("</Movable>");
        code.unindent();
        code.println("</Macro>\n");

        //generates the definition of the red robot
        code.println("<Macro name=\"" + name2 + "-red\">");
        code.indent();
        code.println("<Use macroName=\"" + name2 + "-blue\">");
        code.indent();
        code.println("<SubstituteSurfaces substitute=\"" + name + "-blue\" with=\"" + name + "-red\" />");
        code.println("<SubstituteSurfaces substitute=\"" + name + "-blue-logo\" with=\"" + name + "-red-logo\" />");
        code.unindent();
        code.println("</Use>");
        code.unindent();
        code.println("</Macro>\n");

        code.unindent();
    }
}
