import java.lang.Math;

/**
 * The class represents a servo.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Servo extends ObjectNode {
    /** The maximum velocity. */
    double maxVelocity;
    
    /** The maximum force. */
    double maxForce;
    
    /** The minimal postion. */
    double minPosition;
    
    /** The maximum position. */
    double maxPosition;

    /**
     * The constructor
     * 
     * @param name The name of the servo.
     * @param parent The reference to the parent object.
     */
    Servo(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates the code for a servo.
     * 
     * @param code The code tree object.
     */
    public void generateCode(TreeStream code) {
        boolean sphere = translation == null || translation.length != 3 || children.peekFirst() != null && children.peekFirst() instanceof Camera;
        if(sphere) {
            Sphere.generateCodeSphereStart(code, false, name, 0.01);
            Physics.generateCodePhysicsDefault(code, 0.001);
            code.indent();
            code.println("<Elements>");
        }

        code.indent();
        code.println("<Hinge name=\"" + name + "\">");

        code.indent();
        if(translation != null && translation.length == 3)
            code.println("<AnchorPoint x=\"" + translation[0] + "\" y=\"" + translation[1] + "\" z=\"" + translation[2] + "\" />");
        else
            code.println("<AnchorPoint />");

        if(rotation != null && rotation.length >= 3)
            code.println("<Axis x=\"" + rotation[0] + "\" y=\"" + rotation[1] + "\" z=\"" + rotation[2] + "\">");
        else
            code.println("<Axis x=\"0.0\" y=\"0.0\" z=\"1.0\">");

        code.indent();
        code.println("<Deflection min=\"" + Math.round(minPosition * 180 / Math.PI) + "\" max=\"" + Math.round(maxPosition * 180 / Math.PI) + "\" />");
        code.println("<AngularMotor name=\"" + name + "\" maxVelocity=\"" + Math.round(maxVelocity * 180 / Math.PI) + "\" maxForce=\"" + maxForce + "\">");
        code.indent();
        code.println("<PController pControlFactor=\"10.0\" />");
        code.unindent();
        code.println("</AngularMotor>");
        code.unindent();
        code.println("</Axis>");
        code.unindent();

        super.generateCode(code);

        code.println("</Hinge>");
        code.unindent();

        if(sphere) {
            code.println("</Elements>");
            code.unindent();
            Sphere.generateCodeSphereEnd(code, false);
        }
    }
}
