import java.util.LinkedList;
/**
 * The class represents the SimRobot structure and helps to build the correct syntax. 
 *
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
public class SimRobotTree
{
    /** Contains all defined appearances */
    static protected LinkedList<Appearance> appearance = new LinkedList<Appearance>();
    
    /** Contains all defined shapes */
    static protected LinkedList<Shape> shapes = new LinkedList<Shape>();
    
    /** Contains all defined robots */
    static protected LinkedList<Robot> robot = new LinkedList<Robot>();
    
    /** Contains all defined coordinates */
    static protected LinkedList<Coordinate> coords = new LinkedList<Coordinate>();

    /**
     * The constructor
     */
    public SimRobotTree() {
    }

    /**
     * The method generates the initial code for a RoSiML include file
     * 
     * @param code The code stream object
     */
    void generateCode(TreeStream code) {
        code.println("<RoSiIncludeFile>");
        code.println("<!-- code has been generated: B-Human Webots-Converter -->\n");
        for(Appearance a : appearance)
            a.generateCode(code);
        for(Coordinate c : coords)
            c.generateCode(code);
        for(Shape s : shapes)
            if(s.geometry instanceof IndexedFaceSet)
                ((IndexedFaceSet)s.geometry).generateCode(code);
        for(Robot r : robot)
            r.generateCode(code);

        code.println("</RoSiIncludeFile>");
    }

    /**
     * The method generates the initial code for a RoSi2 include file
     * 
     * @param code The code stream object
     */
    void generateRoSi2CodeRoot(TreeStream code) {
        code.println("<Simulation>");
        code.println("<!-- code has been generated: B-Human Webots-Converter -->\n");
        for(Appearance a : appearance)
            a.generateRoSi2Code(code);
        for(Coordinate c : coords)
            c.generateRoSi2Code(code);
        for(Shape s : shapes)
            if(s.geometry instanceof IndexedFaceSet)
                ((IndexedFaceSet)s.geometry).generateRoSi2Code(code);
        for(Robot r : robot)
            r.generateRoSi2Code(code);

        code.println("</Simulation>");
    }
    void generateRoSi2Code(TreeStream code) {
        // sorry, not enough time to add generateRoSi2Code to all derived classes of SimRobotTree
        // TODO: add generateRoSi2Code to all derived classes of SimRobotTree
    }
}