/**
 * The clas represents a sphere.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Sphere extends Geometry {
    /** The radius of the sphere. */
    double radius;
    
    /** The counte to build a unique name. */
    private static int count = 0;

    /**
     * The constructor.
     * 
     * @param name The sphere identifier name.
     * @param parent The reference to the parent object.
     */
    Sphere(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates a unique name.
     */
    public void generateName() {
        if(name != null)
            return;
        generateName("SPHERE" + count++);
    }

    /**
     * The method generates the bounding code begin.
     * 
     * @param code The code output stream.
     * @param simple Is it a simple physical object.
     */
    public void generateCodeBoundingStart(TreeStream code, boolean simple) {
        code.indent();
        code.println("<" + (simple ? "Simple" : "") + "Sphere name=\"" + name + "\" radius=\"" + radius + "\">");
    }

    /**
     * The method generates the bounding code closure.
     * 
     * @param code The code output stream.
     * @param simple Is it a simple physical object.
     */
    public void generateCodeBoundingEnd(TreeStream code, boolean simple) {
        Sphere.generateCodeSphereEnd(code, simple);
    }

    /**
     * The method generates a sphere code begin with specific name and radius.
     * 
     * @param code The code output stream.
     * @param simple Is it a simple physical object?
     * @param name The sphere identifier name.
     * @param radius The radius of the sphere.
     */
    static public void generateCodeSphereStart(TreeStream code, boolean simple, String name, double radius) {
        code.indent();
        code.println("<" + (simple ? "Simple" : "") + "Sphere name=\"" + name + "_SPHERE\" radius=\"" + radius + (simple ? "" : "\" canCollide=\"false") + "\">");
    }

    /**
     * The method generates a sphere code closure.
     *
     * @param code The code output stream.
     * @param simple Is it a simple physical object.
     */
    static public void generateCodeSphereEnd(TreeStream code, boolean simple) {
        code.println("</" + (simple ? "Simple" : "") + "Sphere>");
        code.unindent();
    }

    /**
     * The method generates a sphere.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        generateCodeBoundingStart(code, false);
        generateCodeBoundingEnd(code, false);
    }
}
