/**
 * The class represents a symbol that was identified in the lexical analysis.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class Symbol extends Position {
    /** All defined symbols. */
    enum Id {
        IDENT, NUMBER, STRING,
        PROTO,
        TRUE, FALSE,
        ROBOT_TYPE, TRANSLATION, ROTATION, NAME,
        CHILDREN, DEF, SHAPE_TYPE, APPEARANCE, APPEARANCE_TYPE,
        MATERIAL, MATERIAL_TYPE, AMBIENT_INTENSITY,
        DIFFUSE_COLOR, SHININESS, SPECULAR_COLOR,
        GEOMETRY, INDEXED_FACE_SET_TYPE, COORD, COORD_TYPE,
        POINT, COORD_INDEX, CREASE_ANGLE,
        USE, TEX_COORD,
        TEX_COORD_TYPE, TEX_COORD_INDEX,
        LED_TYPE, EMISSIVE_COLOR, COLOR, GRADUAL,
        ACCELEROMETER_TYPE, LOOKUP_TABLE,
        GYRO_TYPE, Z_AXIS,
        DISTANCE_SENSOR_TYPE, TYPE, NUMBER_OF_RAYS, APERTURE,
        SERVO_TYPE, MAX_VELOCITY, MAX_FORCE, MIN_POSITION, MAX_POSITION,
        TRANSFORM_TYPE,
        CYLINDER_TYPE, BOTTOM, HEIGHT, RADIUS, TOP, SUBDIVISION,
        CAMERA_TYPE, WINDOW_POSITION, PIXEL_SIZE, FIELD_OF_VIEW,
        WIDTH, NEAR,
        TEXTURE, IMAGE_TEXTURE_TYPE, URL,
        BOUNDING_OBJECT, SPHERE_TYPE, PHYSICS,
        PHYSICS_TYPE, DENSITY, MASS, CENTER_OF_MASS,
        BOX_TYPE, SIZE, TOUCH_SENSOR_TYPE,
        LBRACKET, RBRACKET,
        LBRACE, RBRACE,
        COMMA,
        EOF
    };

    /** The symbol. */
    Id id;

    /** If the symbol is type of FLOAT, the read number is stored here. */
    double number;

    /** If the symbol is type of IDENT or STRING, the read identifier is stored here. */
    String ident;

    /**
     * The constructor.
     * @param id The identified symbol.
     * @param position The position in source code where the symbol was identified.
     */
    Symbol(Id id, Position position) {
        super(position.line, position.column);
        this.id = id;
    }

    /**
     * This method creates a printable string from this object.
     * @return The string.
     */
    public String toString() {
        switch (id) {
        case IDENT:
            return "IDENT: " + ident;
        case NUMBER:
            return "NUMBER: " + number;
        case STRING:
            return "STRING: " + ident;
        default:
            return id.toString();
        }
    }
}
