import java.util.LinkedList;

/**
 * The class represents a touch sensor.
 * 
 * @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R&ouml;fer</a>
 * @author <a href="mailto:b21@informatik.uni-bremen.de">Oliver B&ouml;sche</a>
 * @author <a href="mailto:raider@informatik.uni-bremen.de">Thiemo Wiedemeyer</a>
 */
class TouchSensor extends ObjectNode {
    /** The sensor response lookup table */
    LinkedList<double[]> lookupTable = new LinkedList<double[]>();
    
    /** The type of sensor (bumper or force) */
    String type;

    /**
     * The constructor.
     * 
     * @param name The sensor identifier name.
     * @param parent The reference to the parent object.
     */
    TouchSensor(String name, Node parent) {
        super(name, parent);
    }

    /**
     * The method generates the code for the sensor type of force.
     * 
     * @param code The code output stream.
     */
    public void generateCodeFsr(TreeStream code) {
        ((Shape)boundingObject).geometry.generateCodeBoundingStart(code, false);

        double[] rot = rotation;
        rotation = null;
        generateCodeTransform(code);
        rotation = rot;
        Appearance.generateCodeAppearanceRef(code, "MAIN_SKIN_APP");

        if(physics != null)
            physics.generateCode(code);
        else
            Physics.generateCodePhysicsDefault(code, 0.001);

        code.indent();
        code.println("<Elements>");
        code.indent();
        code.println("<PressureSensor name=\"" + name + "\">");
        code.indent();

        code.println("<Gain x=\"0\" y=\"0\" z=\"-4\"/>");
        code.println("<Offset z=\"-0.032\"/>");

        code.println("<Elements>");
        ((Shape)boundingObject).geometry.generateCodeBoundingStart(code, false);
        Transform.generateCodeTransform(code, 0.0, 0.0,  -0.002);
        Appearance.generateCodeAppearanceRef(code, "MAIN_SKIN_APP");

        if(physics != null)
            physics.generateCode(code);
        else
            Physics.generateCodePhysicsDefault(code, 0.001);

        generateCodeBoundingEnd(code, false);
        code.println("</Elements>");
        code.unindent();
        code.println("</PressureSensor>");
        code.unindent();
        code.println("</Elements>");
        code.unindent();
        generateCodeBoundingEnd(code, false);
    }

    /**
     * The method generates the code for sensor type of bumper.
     * 
     * @param code The code output stream.
     */
    public void generateCodeBumper(TreeStream code) {
        code.indent();
        code.println("<Bumper name=\"" + name + "\">");

        generateCodeTransform(code);

        code.indent();
        code.println("<Elements>");

        generateCodeBoundingStart(code, false);

        Appearance.generateCodeAppearanceRef(code, "nao-blue");
        if(physics != null)
            physics.generateCode(code);
        else
            Physics.generateCodePhysicsDefault(code, 0.001);

        generateCodeBoundingEnd(code, false);

        code.println("</Elements>");
        code.unindent();
        code.println("</Bumper>");
        code.unindent();
    }

    /**
     * The method generates the code to represent a type of touch sensor.
     * 
     * @param code The code output stream.
     */
    public void generateCode(TreeStream code) {
        if(boundingObject instanceof Shape && ((Shape)boundingObject).geometry instanceof Sphere)
            generateCodeFsr(code);
        else
            generateCodeBumper(code);
    }
}
