/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../tools/asserts.h"
#include "identifier.h"
#include "contextAnalysisException.h"
#include "declarations/declaration.h"
#include "declarations/moduleDeclaration.h"
#include "declarations/enumDeclaration.h"
#include "declarationCollection.h"

DeclarationCollection::~DeclarationCollection()
{
    ASSERT(declarations.size() == 1);
    ASSERT(forStatementStack.size() == 0);
}

void DeclarationCollection::add(const std::string &name, ast::Declaration *declaration)
{
    DeclarationMap::iterator decl = declarations.front().find(name);
    if(decl != declarations.front().end())
        throw ContextAnalysisException(declaration->ident->location, "multiple declarations of identifier %s", name.c_str());
    declarations.front()[name] = declaration;
}

void DeclarationCollection::stripToTopLevel()
{
    while(declarations.size() > 1)
        declarations.pop_back();
}

bool DeclarationCollection::resolve(ast::ResolvableIdentifier *ident)
{
    DeclarationCollection *collection = this;
    for(int i = ident->path.size() - 1; i >= 0; i--)
    {
      ast::Declaration *declaration = collection->resolve(ident->path.at(i));
      if(!declaration)
        return false;

      if(declaration->getDeclarationType() == ast::Declaration::Module)
      {
        if(!moduleDeclaration || moduleDeclaration->ident->name != ident->path.at(i))
          collection = &((ast::ModuleDeclaration*)declaration)->localDeclarationCollection;
      }
      else if(declaration->getDeclarationType() == ast::Declaration::Enum)
        collection = &((ast::EnumDeclaration*)declaration)->localDeclarationCollection;
      else
        throw ContextAnalysisException(ident->location, "%s is not a module or enum, thus can't be accessed via '::'", ident->path.at(i).c_str());
    }

    ident->declaration = collection->resolve(ident->name);
    return ident->declaration != 0;
}

ast::Declaration* DeclarationCollection::resolve(const std::string &name)
{
    for(DeclarationList::iterator it = declarations.begin(); it != declarations.end(); it++)
    {
        DeclarationMap::iterator decl = it->find(name);
        if(decl != it->end())
          return decl->second;
    }

    return 0;
}

