/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <list>
#include <vector>
#include <string>
#include "../tools/tr1Include.h"

namespace ast
{
class Declaration;
class ResolvableIdentifier;
class ForStatement;
}

class DeclarationCollection
{
    public:
        int varIdInFrame;
        ast::Declaration *functionDeclaration,
                         *moduleDeclaration;
        std::vector<ast::ForStatement*> forStatementStack;

        DeclarationCollection()
            : varIdInFrame(0),
              functionDeclaration(0),
              moduleDeclaration(0)
        {
            enter();
        }

        ~DeclarationCollection();

        void enter()
        {
            declarations.push_front(DeclarationMap());
        }

        void leave()
        {
            declarations.pop_front();
        }

        void clear()
        {
            declarations.clear();
            forStatementStack.clear();
            enter();
        }

        void add(const std::string &name, ast::Declaration *declaration);
        void stripToTopLevel();

        bool resolve(ast::ResolvableIdentifier *ident);
        ast::Declaration* resolve(const std::string &name);

    private:
        typedef std::tr1::unordered_map<std::string, ast::Declaration*> DeclarationMap;
        typedef std::list<DeclarationMap> DeclarationList;
        DeclarationList declarations;
};

