/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../declarationCollection.h"
#include "../identifier.h"
#include "functionDeclaration.h"
#include "variableDeclaration.h"
#include "classDeclaration.h"

using namespace ast;

ClassDeclaration::ClassDeclaration(Identifier *ident)
    : Declaration(ident)
{}

void ClassDeclaration::contextAnalysis(DeclarationCollection &declarationCollection)
{
    declarationCollection.add(ident->name, this);

    declarationCollection.enter();

    for(size_t i = 0; i < memberFunctions.size(); i++)
        memberFunctions.at(i)->contextAnalysis(declarationCollection);
    for(size_t i = 0; i < memberVariables.size(); i++)
        memberVariables.at(i)->contextAnalysis(declarationCollection);

    localDeclarationCollection = declarationCollection;
    localDeclarationCollection.stripToTopLevel();

    declarationCollection.leave();
}

ClassDeclaration::~ClassDeclaration()
{
    for(size_t i = 0; i < memberFunctions.size(); i++)
        delete memberFunctions.at(i);
    memberFunctions.clear();
    for(size_t i = 0; i < memberVariables.size(); i++)
        delete memberVariables.at(i);
    memberVariables.clear();

    localDeclarationCollection.clear();
}

