/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <vector>
#include <string>
#include "../declarationCollection.h"
#include "declaration.h"

class CodeGeneratorInterface;
class Stack2;

namespace ast
{

class FunctionDeclaration;
class ClassDeclaration;
class EnumDeclaration;
class GlobalVariableDeclaration;
class ValueDeclarationBase;

class ModuleDeclaration : public Declaration
{
    public:
        std::vector<FunctionDeclaration*> functionDeclarations; //this also contains the tasks!
        std::vector<std::string> requirements;
        std::vector<ValueDeclarationBase*> valueDeclarations;

        DeclarationCollection localDeclarationCollection;

        ModuleDeclaration(const char *name);
        virtual ~ModuleDeclaration();

        DeclarationType getDeclarationType() { return Module; }

        virtual void contextAnalysis(DeclarationCollection &declarationCollection);
        virtual void contentContextAnalysis(DeclarationCollection &declarationCollection);
        virtual void execute(Stack2 &stack) {}
        virtual void generateCode(CodeGeneratorInterface &cg);
};

class ExternModuleDeclaration : public ModuleDeclaration
{
    public:
        ExternModuleDeclaration(const char *name)
            : ModuleDeclaration(name)
        {}

        virtual ~ExternModuleDeclaration();

        std::vector<ClassDeclaration*> classDeclarations;
        std::vector<EnumDeclaration*> enumDeclarations;

        virtual void contentContextAnalysis(DeclarationCollection &declarationCollection);
        virtual void execute(Stack2 &stack) {}
        virtual void generateCode(CodeGeneratorInterface &cg);
};

class MasterModuleDeclaration : public ExternModuleDeclaration
{
    public:
        std::vector<ModuleDeclaration*> moduleDeclarations;
        Statement* mainCallStatement;
        ModuleDeclaration *mainModule;
        GlobalVariableDeclaration *inputDeclaration,
                                  *outputDeclaration;

        MasterModuleDeclaration();
        virtual ~MasterModuleDeclaration();

        virtual void contentContextAnalysis(DeclarationCollection &declarationCollection);
        virtual void generateCode(CodeGeneratorInterface &cg);
        virtual void execute(Stack2 &stack);
};

}

