/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "../../../tools/asserts.h"
#include "../../../stack.h"
#include "../classDeclaration.h"
#include "typeToName.h"
#include "funcWrappers.h"
#include "varWrappers.h"

class ClassWrapperBase : public ast::ClassDeclaration
{
    public:
        ClassWrapperBase(ast::Identifier *identifier)
            : ast::ClassDeclaration(identifier)
        {}

        template<typename F>
        ClassWrapperBase& func(const char *name, F f, const char *cppName = "")
        {
            FuncWrapper *wrapper = new FuncWrapper(new ast::Identifier(name, Location::builtInLocation, cppName), new Functor<F>(f));
            memberFunctions.push_back(wrapper);
            return *this;
        }

        template<typename F>
        ClassWrapperBase& funcVoid(const char *name, F f, const char *cppName = "")
        {
            FuncWrapper *wrapper = new FuncWrapper(new ast::Identifier(name, Location::builtInLocation, cppName), new FunctorVoid<F>(f));
            memberFunctions.push_back(wrapper);
            return *this;
        }

        template<typename V>
        ClassWrapperBase& var(const char *name, V v, const char *cppName = "")
        {
            VarWrapperBase *wrapper = new VarWrapper<V>(new ast::Identifier(name, Location::builtInLocation, cppName), v);
            memberVariables.push_back(wrapper);
            return *this;
        }
};

template<typename T>
class ClassWrapper : public ClassWrapperBase
{
    public:
        ClassWrapper(const std::string &moduleName, ast::Identifier *identifier)
            : ClassWrapperBase(identifier)
        {
            TypeToName::registerType<T>(moduleName, identifier->name);
        }

        inline void createInstance(Stack2 &stack, bool callCtor)
        {
            void *ptr = stack.push<T>(callCtor);
            ASSERT(ptr);
            stack.pushClassHandler(this);
            stack.pushPtr(ptr);
        }

        inline void deleteInstance(Stack2 &stack)
        {
            stack.pop<T>();
#ifdef DEBUG
            ASSERT(stack.popClassHandler() == this);
#else
            stack.popClassHandler();
#endif
        }

        inline void assign(void *d, void *s)
        {
            *(T*)d = *(T*)s;
        }

        void execute(Stack2 &stack) {}
};

template<typename T>
class ClassWrapper<const T&> : public ClassWrapper<T> {};

template<typename T>
ClassWrapperBase& registerClass(ast::ExternModuleDeclaration *module,
                                const char *name,
                                const std::string &cppName= "")
{
    ClassWrapperBase *wrapper = new ClassWrapper<T>(module->ident->name, new ast::Identifier(name, Location::builtInLocation, cppName));
    module->classDeclarations.push_back(wrapper);
    return *wrapper;
}

