/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <typeinfo>
#include <string>
#include <exception>
#include "../../../tools/asserts.h"
#include "../../../tools/tr1Include.h"

class TypeToName
{
    public:
        class TypeNotRegisteredException : public std::exception
        {
            public:
                TypeNotRegisteredException(const std::string &msg);

                ~TypeNotRegisteredException() throw();

                virtual const char* what() const throw();
            private:
                std::string msg;
        };

        typedef std::tr1::unordered_map<std::string, std::pair<std::string, std::string> > TypeToNameMap;

        template<typename T>
        static void registerType(const std::string &moduleName, const std::string &name)
        {
            TypeToNameMap::mapped_type p(moduleName, name);
            typeToName.insert(TypeToNameMap::value_type(typeid(T).name(), p));
        }

        template<typename T>
        static const char* getName()
        {
            const TypeToNameMap::mapped_type *t = getTypeInfo<T>();
            return t ? t->second.c_str() : 0;
        }

        template<typename T>
        static const char* getModuleName()
        {
            const TypeToNameMap::mapped_type *t = getTypeInfo<T>();
            return t ? t->first.c_str() : 0;
        }

        template<typename T>
        static const TypeToNameMap::mapped_type* getTypeInfo()
        {
            return getTypeInfo(typeid(T).name());
        }

        static const TypeToNameMap::mapped_type* getTypeInfo(const char *typeIdName);

    private:
        static TypeToNameMap typeToName;
};

