/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <string.h>
#include "../../../stack.h"
#include "../../identifier.h"
#include "../../contextAnalysisException.h"
#include "../variableDeclaration.h"
#include "typeToName.h"

class VarWrapperBase : public ast::VariableDeclaration
{
    public:
        VarWrapperBase(ast::Identifier *identifier)
            : VariableDeclaration(identifier, 0, 0)
        {}

        void execute(Stack2 &stack) {}
        void executeCleanUp(Stack2 &stack) {}
};

template<typename V>
class VarWrapper {};

template<typename T, typename C>
class VarWrapper<T C::*> : public VarWrapperBase
{
    public:
        typedef T C::*var_type;
        var_type v;
        VarWrapper(ast::Identifier *identifier, var_type v)
            : VarWrapperBase(identifier),
              v(v)
        {}

        virtual void contextAnalysis(DeclarationCollection &declarationCollection)
        {
            ASSERT(!typeIdent);
            try
            {
                typeIdent = new ast::ResolvableIdentifier(TypeToName::getName<T>(), Location::builtInLocation);
            }
            catch(TypeToName::TypeNotRegisteredException &e)
            {
                throw ContextAnalysisException(getLocation(), e.what());
            }

            const char *moduleName = TypeToName::getModuleName<T>();
            if(strcmp(moduleName, "_master_"))
                typeIdent->pushPath(moduleName);

            VarWrapperBase::contextAnalysis(declarationCollection);
        }

        void call(Stack2 &stack)
        {
            void *p = stack.popPtr();
            void *p2 = &(((C*)p)->*v);
            stack.pushPtr(p2);
        }
};

