/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>

#include "../../stack.h"
#include "../../executionContext.h"
#include "../../callTree.h"
#include "../../tools/asserts.h"
#include "../declarationCollection.h"
#include "../identifier.h"
#include "../statements/statementBlock.h"
#include "taskDeclaration.h"

using namespace ast;

TaskDeclarationBase::TaskDeclarationBase(Identifier *ident,
                                         FunctionDeclaration::ParameterDeclaration *paramDecls,
                                         StatementBlock *statements,
                                         const Location &location)
    : FunctionDeclaration(ident,
                          0,
                          paramDecls,
                          statements,
                          location),
      state(restart),
      lastExecution(0)
{}

TaskDeclaration::TaskDeclaration(Identifier *ident,
                                 FunctionDeclaration::ParameterDeclaration *paramDecls,
                                 StatementBlock *statements,
                                 const Location &location)
    : TaskDeclarationBase(ident,
                          paramDecls,
                          statements,
                          location),
      internalState(0)
{
    stack.owner = this;
}

TaskDeclaration::~TaskDeclaration()
{
    stack.clear();
}

void TaskDeclaration::call(Stack2 &stack)
{
    PUSH_CALL_TREE(stack.executionContext->callTree, ident->name);

    if(stack.executionContext->frameID - lastExecution > 1)
        state = TaskDeclaration::restart;
    lastExecution = stack.executionContext->frameID;

    this->stack.executionContext = stack.executionContext;
    if(state <= TaskDeclarationBase::done)
    {
        this->stack.clear();
        for(size_t i = 0; i < paramDecls->size(); i++)
            this->stack.pushPtr(0);
        state = restart;
    }

#ifdef DEBUG
    if(state == TaskDeclarationBase::restart)
        ASSERT(this->stack.ptrStackSize() == paramDecls->size());
    else
        ASSERT(this->stack.ptrStackSize() >= paramDecls->size());
#endif

    for(size_t i = 0; i < paramDecls->size(); i++)
        this->stack.setAbsPtr(paramDecls->size() - 1 - i, stack.popPtr());

    call();

    POP_CALL_TREE(stack.executionContext->callTree);
}

void TaskDeclaration::call()
{
    if(state == restart)
        internalState = 0;

    switch(internalState)
    {
        case 0:
        callTreeStackSize = stack.executionContext->callTree.stackSize();
        stack.pushFrame();

        internalState = 1;
        case 1:
        try
        {
            statements->execute(stack);
            if(stack.interupt)
            {
                ASSERT(stack.interupt == Stack2::Yield);
                stack.interupt = Stack2::None;
                return;
            }
        }
        catch(StackException &e)
        {
            stack.executionContext->errorHandler("ERROR: Stack error in task %s Message: %s\nLocation:%s", ident->name.c_str(), e.msg.c_str(), getLocation().toString().c_str());
            stack.executionContext->callTree.trace("!!!Stack error!!!");
            while(stack.executionContext->callTree.stackSize() > callTreeStackSize)
                stack.executionContext->callTree.popCall();
            state = TaskDeclarationBase::restart;
            break;
        }

        stack.popFrame();
        state = TaskDeclarationBase::done;
        internalState = 0;
    }
}

