/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <vector>
#include "../../stack.h"
#include "functionDeclaration.h"

class CodeGeneratorInterface;
class DeclarationCollection;
class Stack2;
namespace yy
{
    class location;
}

namespace ast
{

class StatementBlock;
class Identifier;

class TaskDeclarationBase : public FunctionDeclaration
{
    public:
        enum State
        {
            //...user defined failure states... (< -1)
            failure=-1,
            done=0,
            restart=done,
            running=1,
            //...user defined running states... (> 1)
        };

        State state;
        unsigned lastExecution;

        TaskDeclarationBase(Identifier *ident,
                            FunctionDeclaration::ParameterDeclaration *paramDecls,
                            StatementBlock *statements,
                            const Location &location);

        virtual DeclarationType getDeclarationType() { return Task; }
        virtual bool isNativeTask() { return false; }
};

class TaskDeclaration : public TaskDeclarationBase
{
    public:
        Stack2 stack;
        int internalState;
        unsigned callTreeStackSize;

        TaskDeclaration(Identifier *ident,
                        FunctionDeclaration::ParameterDeclaration *paramDecls,
                        StatementBlock *statements,
                        const Location &location);
        virtual ~TaskDeclaration();

        virtual void call();
        virtual void call(Stack2 &stack);
        virtual void generateCode(CodeGeneratorInterface &cg);
};

}

