/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../contextAnalysisException.h"
#include "../declarationCollection.h"
#include "valueDeclaration.h"

using namespace ast;

ValueDeclarationBase::ValueDeclarationBase(Identifier *identifier, ResolvableIdentifier *typeIdent)
  : Declaration(identifier),
    typeIdent(typeIdent)
{
    ASSERT(typeIdent);
}

ValueDeclarationBase::~ValueDeclarationBase()
{
    delete typeIdent;
}

void ValueDeclarationBase::contextAnalysis(DeclarationCollection &declarationCollection)
{
    declarationCollection.add(ident->name, this);

    if(!declarationCollection.resolve(typeIdent))
        throw ContextAnalysisException(getLocation(), "can't resolve %s",
                                       typeIdent->fullName().c_str());
    else if(typeIdent->declaration->getDeclarationType() != Declaration::Class)
        throw ContextAnalysisException(getLocation(),
                                       "%s resolved to non class declaration",
                                       typeIdent->fullName().c_str());
}

