/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <string>

#include "../../stack.h"
#include "native/typeToName.h"
#include "../identifier.h"

namespace ast
{

class ValueDeclarationBase : public Declaration
{
    public:
        ResolvableIdentifier *typeIdent;

        ValueDeclarationBase(Identifier *identifier, ResolvableIdentifier *typeIdent);
        virtual ~ValueDeclarationBase();

        virtual void contextAnalysis(DeclarationCollection &declarationCollection);
        virtual Declaration::DeclarationType getDeclarationType() { return Declaration::Value; }
        virtual void generateCode(CodeGeneratorInterface &cg) { ASSERT(false); }
        virtual void execute(Stack2 &stack) {}
        virtual void call(Stack2 &stack) = 0;
};

template<typename T>
class ValueDeclaration : public ValueDeclarationBase
{
    public:
        const T value;

        ValueDeclaration(Identifier *identifier, T value)
          : ValueDeclarationBase(identifier,
                                 new ResolvableIdentifier(TypeToName::getName<T>(),
                                                          Location::builtInLocation)),
            value(value)
        {}

        void call(Stack2 &stack)
        {
            stack.pushPtr((void*)&value);
            //HACK! Watchout, this is a const cast
            //the pushed void* is not const! This means the "engine" could(!) modify 
            //value! This shoulder never happen! It could be checked with an assert,
            //but......... I hope it will just never ever happen
        }
};

}

