/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include <limits.h>
#include "../../stack.h"
#include "../../tools/asserts.h"
#include "../identifier.h"
#include "../declarationCollection.h"
#include "../contextAnalysisException.h"
#include "classDeclaration.h"
#include "variableDeclaration.h"

using namespace ast;

VariableDeclaration::VariableDeclaration(Identifier *ident,
                                         ResolvableIdentifier *typeIdent,
                                         Statement *initializeStatement,
                                         char qualifiers)
    : Declaration(ident),
      typeIdent(typeIdent),
      initializeStatement(initializeStatement),
      qualifiers(qualifiers),
      idInFrame(UINT_MAX)
{
    ASSERT(ident);
}

VariableDeclaration::~VariableDeclaration()
{
    if(typeIdent)
        delete typeIdent;
    if(initializeStatement)
        delete initializeStatement;
}

void VariableDeclaration::contextAnalysis(DeclarationCollection &declarationCollection)
{
    ASSERT(typeIdent);
    idInFrame = declarationCollection.varIdInFrame++;

    declarationCollection.add(ident->name, this);


    if(!declarationCollection.resolve(typeIdent))
        throw ContextAnalysisException(typeIdent->location,
                                       "can't resolve type %s",
                                       typeIdent->fullName().c_str());
    else if(typeIdent->declaration->getDeclarationType() != Declaration::Class)
        throw ContextAnalysisException(typeIdent->location,
                                       "identifier declaration of '%s' is not a class declaration",
                                       typeIdent->fullName().c_str());

    if(initializeStatement)
        initializeStatement->contextAnalysis(declarationCollection);
}

void VariableDeclaration::execute(Stack2 &stack)
{
    ((ClassDeclaration*)typeIdent->declaration)->createInstance(stack, true);
    //this should only happen in for-loops! All other initializeStatement are
    //moved to the statements list of the statementblock
    if(initializeStatement)
    {
        initializeStatement->execute(stack);
        initializeStatement->executeCleanUp(stack);
    }
}

void VariableDeclaration::executeCleanUp(Stack2 &stack)
{
    stack.popPtr();
    ((ClassDeclaration*)typeIdent->declaration)->deleteInstance(stack);
}

void VariableDeclaration::call(Stack2 &stack)
{
    void *p = stack.getFramePtr(idInFrame);
    stack.pushPtr(p);
}

void GlobalVariableDeclaration::call(Stack2 &stack)
{
    stack.pushPtr(externPtr);
}

