/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "declaration.h"

class CodeGeneratorInterface;
class DeclarationCollection;
namespace yy
{
    class location;
}

namespace ast
{

class ResolvableIdentifier;
class Identifier;
class Statement;

class VariableDeclaration : public Declaration
{
    public:
        ResolvableIdentifier *typeIdent;
        Statement *initializeStatement;
        enum
        {
            writable = 0x01,
            readable = 0x02,
        };
        char qualifiers;
        int idInFrame;

        enum VariableType
        {
            Local,
            Global,
        };

        VariableDeclaration(Identifier *ident,
                            ResolvableIdentifier *typeIdent,
                            Statement *initializeStatement,
                            char qualifiers = writable | readable);
        virtual ~VariableDeclaration();

        virtual DeclarationType getDeclarationType() { return Variable; }
        virtual VariableType getVariableType() { return Local; }

        virtual void contextAnalysis(DeclarationCollection &declarationCollection);
        virtual void execute(Stack2 &stack);
        virtual void executeCleanUp(Stack2 &stack);
        virtual void call(Stack2 &stack);
        virtual void generateCode(CodeGeneratorInterface &cg);
};

class GlobalVariableDeclaration : public VariableDeclaration
{
    public:
        GlobalVariableDeclaration(Identifier *ident,
                                  ResolvableIdentifier *typeIdent,
                                  void *ptr,
                                  char qualifiers = writable | readable)
            : VariableDeclaration(ident,
                                  typeIdent,
                                  0,
                                  qualifiers),
              externPtr(ptr)
        {}

        virtual VariableType getVariableType() { return Global; }
        virtual void call(Stack2 &stack);
        virtual void generateCode(CodeGeneratorInterface &cg);

        void *externPtr;
};

}

