/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "expression.h"

namespace ast
{

class ClassDeclaration;
class Declaration;
class MemberAccessExpression;

class BinaryExpression : public Expression
{
    public:
        typedef enum
        {
            plus,
            minus,
            mul,
            div,
            mod,
            lt,
            leq,
            gt,
            geq,
            eq,
            neq,
            and_,
            or_,
        } Type;

        static const char *operationNames[];

        typedef enum
        {
            bool_,
            int_,
            float_,
            other,
        } DataType;

        Type type;
        DataType dataType;
        Expression *leftExpr, *rightExpr;
        ClassDeclaration *typeDecl;
        MemberAccessExpression *customOpCall;

        BinaryExpression(Type type, Expression *leftExpr, Expression *rightExpr);
        ~BinaryExpression();

        Declaration* getType() { return (Declaration*)typeDecl; }
        virtual void contextAnalysis(DeclarationCollection &declarationCollection);
        virtual void execute(Stack2 &stack);
        virtual void executeCleanUp(Stack2 &stack);
        virtual bool isLValue() { return false; }
        virtual bool isRValue() { return leftExpr->isRValue() && rightExpr->isRValue(); }
        virtual void generateCode(CodeGeneratorInterface &cg);
};

}

