/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../../tools/asserts.h"
#include "../declarationCollection.h"
#include "../identifier.h"
#include "../contextAnalysisException.h"
#include "../declarations/variableDeclaration.h"
#include "../declarations/valueDeclaration.h"
#include "identifierAccessExpression.h"

using namespace ast;

IdentifierAccessExpression::IdentifierAccessExpression(ResolvableIdentifier *ident)
    : Expression(ident->location),
      ident(ident)
{
    ASSERT(ident);
}

IdentifierAccessExpression::~IdentifierAccessExpression()
{
    delete ident;
}

Declaration* IdentifierAccessExpression::getType()
{
    if(ident->declaration->getDeclarationType() == Declaration::Variable)
        return ((VariableDeclaration*)ident->declaration)->typeIdent->declaration;
    else if(ident->declaration->getDeclarationType() == Declaration::Value)
    {
        ValueDeclarationBase *decl = (ValueDeclarationBase*)ident->declaration;
        return decl->typeIdent->declaration;
    }

    ASSERT(false);
    return 0;
}

bool IdentifierAccessExpression::isLValue()
{
    if(ident->declaration->getDeclarationType() == Declaration::Variable)
        return ((VariableDeclaration*)ident->declaration)->qualifiers & VariableDeclaration::writable;
    else if(ident->declaration->getDeclarationType() == Declaration::Value)
        return false;

    ASSERT(false);
    return false;
}

bool IdentifierAccessExpression::isConst()
{
    if(ident->declaration->getDeclarationType() == Declaration::Variable)
        return !(((VariableDeclaration*)ident->declaration)->qualifiers & VariableDeclaration::writable);
    else if(ident->declaration->getDeclarationType() == Declaration::Value)
        return true;

    ASSERT(false);
    return false;
}

bool IdentifierAccessExpression::isRValue()
{
    if(ident->declaration->getDeclarationType() == Declaration::Variable)
        return (((VariableDeclaration*)ident->declaration)->qualifiers & VariableDeclaration::readable) != 0;
    else if(ident->declaration->getDeclarationType() == Declaration::Value)
        return true;

    ASSERT(false);
    return false;
}

void IdentifierAccessExpression::contextAnalysis(DeclarationCollection &declarationCollection,
                                                 DeclarationCollection &memberDeclarationCollection)
{
    contextAnalysis(memberDeclarationCollection);
}

void IdentifierAccessExpression::contextAnalysis(DeclarationCollection &declarationCollection)
{
    if(!declarationCollection.resolve(ident))
        throw ContextAnalysisException(ident->location,
                                       "can't resolve identifier '%s'",
                                       ident->fullName().c_str());
    else if(   ident->declaration->getDeclarationType() != Declaration::Variable
            && ident->declaration->getDeclarationType() != Declaration::Value)
        throw ContextAnalysisException(ident->location,
                                       "expected variable or enumvalue name instead of '%s'",
                                       ident->fullName().c_str());
}

void IdentifierAccessExpression::execute(Stack2 &stack)
{
    if(ident->declaration->getDeclarationType() == Declaration::Variable)
    {
        VariableDeclaration *decl = (VariableDeclaration*)ident->declaration;
        decl->call(stack);
    }
    else if(ident->declaration->getDeclarationType() == Declaration::Value)
    {
        ValueDeclarationBase *decl = (ValueDeclarationBase*)ident->declaration;
        decl->call(stack);
    }
}

