/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../declarationCollection.h"
#include "../identifier.h"
#include "../declarations/declaration.h"
#include "../contextAnalysisException.h"
#include "literalExpressions.h"

using namespace ast;

LiteralExpressionBase::LiteralExpressionBase(ResolvableIdentifier *typeIdent)
    : Expression(typeIdent->location),
      typeIdent(typeIdent)
{}
LiteralExpressionBase::~LiteralExpressionBase()
{
    delete typeIdent;
}

Declaration* LiteralExpressionBase::getType()
{
    ASSERT(typeIdent->declaration);
    return typeIdent->declaration;
}

void LiteralExpressionBase::contextAnalysis(DeclarationCollection &declarationCollection)
{
    if(!declarationCollection.resolve(typeIdent))
        throw ContextAnalysisException(getLocation(),
                                       "can't resolve type '%s'",
                                       typeIdent->fullName().c_str());
    else if(typeIdent->declaration->getDeclarationType() != Declaration::Class)
        throw ContextAnalysisException(getLocation(),
                                       "type identifier '%s' resolved to non class declaration",
                                       typeIdent->fullName().c_str());
}

