/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "../../stack.h"

#include "expression.h"

class DeclarationCollection;
class CodeGeneratorInterface;

namespace ast
{

class ResolvableIdentifier;
class ClassDeclaration;

class LiteralExpressionBase : public Expression
{
    public:
        ResolvableIdentifier *typeIdent;

        LiteralExpressionBase(ResolvableIdentifier *typeIdent);

        ~LiteralExpressionBase();

        Declaration* getType();

        virtual void contextAnalysis(DeclarationCollection &declarationCollection);
        virtual bool isLValue() { return false; }

};

template <typename T>
class LiteralExpression : public LiteralExpressionBase
{
    /*
     * hmmmm I don't know whether it's a good idea to handle
     * literal expressions with a template class. But for now
     * I don't want to maintain x seperate LiteralXXXClasses.
     * Let's see whether it'll survive.
     */

    public:
        const T value;

        LiteralExpression(ResolvableIdentifier *typeIdent, T v)
            : LiteralExpressionBase(typeIdent),
              value(v)
        {}

        virtual void execute(Stack2 &stack)
        {
            stack.pushPtr((void*)&value);
            //HACK! const cast.... value could be modified by the engine, but
            //it should never happen
        }

        virtual void executeCleanUp(Stack2 &stack) {}

        virtual void generateCode(CodeGeneratorInterface &cg);
};

}

