/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../../stack.h"
#include "../declarationCollection.h"
#include "../identifier.h"
#include "../contextAnalysisException.h"
#include "../declarations/classDeclaration.h"
#include "../declarations/taskDeclaration.h"
#include "../declarations/moduleDeclaration.h"

#include "taskStateExpression.h"

using namespace ast;

TaskStateExpression::TaskStateExpression(ResolvableIdentifier *taskIdent)
    : Expression(Location::builtInLocation),
      taskIdent(taskIdent),
      intDeclaration(0)
{
    ASSERT(taskIdent);
}

TaskStateExpression::~TaskStateExpression()
{
    delete taskIdent;
}

Declaration* TaskStateExpression::getType()
{
    return intDeclaration;
}

void TaskStateExpression::contextAnalysis(DeclarationCollection &declarationCollection)
{
    ResolvableIdentifier intIdent("int", Location::builtInLocation);
    if(!declarationCollection.resolve(&intIdent))
        throw ContextAnalysisException(getLocation(), "can't resolve int!");

    intDeclaration = intIdent.declaration;

    if(!declarationCollection.resolve(taskIdent))
        throw ContextAnalysisException(taskIdent->location,
                                       "can't resolve %s\n",
                                       taskIdent->fullName().c_str());
    else if(taskIdent->declaration->getDeclarationType() != Declaration::Task)
        throw ContextAnalysisException(taskIdent->location,
                                       "task identifier '%s' resolved to non task declaration",
                                       taskIdent->fullName().c_str());
}

void TaskStateExpression::execute(Stack2 &stack)
{
    ((ClassDeclaration*)intDeclaration)->createInstance(stack);

    int &result = *(int*)stack.popPtr();

    TaskDeclarationBase *taskDecl = (TaskDeclarationBase*)taskIdent->declaration;
    result = taskDecl->state;

    stack.pushPtr(&result);
}

void TaskStateExpression::executeCleanUp(Stack2 &stack)
{
    ((ClassDeclaration*)intDeclaration)->deleteInstance(stack);
}

