/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <string>
#include <vector>

#include "../parser/location.h"

namespace ast
{

class Declaration;

class Identifier
{
    public:
        std::string name;
        std::string cppName;
        Location location;

        Identifier(const std::string &name, const Location &location, const std::string &cppName = "")
            : name(name),
              cppName(cppName),
              location(location)
        {}
};

class ResolvableIdentifier : public Identifier
{
    public:
        Declaration *declaration;
        std::vector<std::string> path;

        ResolvableIdentifier(const std::string &name, const Location &location)
            : Identifier(name, location),
              declaration(0)
        {}

        std::string toString();

        void pushPath(const std::string &subpath)
        {
          path.push_back(subpath);
        }

        std::string fullName()
        {
          std::string t;
          for(int i = path.size()-1; i >= 0; i--)
            t += path.at(i) + "::";
          t += name;
          return t;
        }
};

}

