/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../../tools/asserts.h"
#include "../../stack.h"
#include "../contextAnalysisException.h"
#include "../declarations/classDeclaration.h"
#include "../expressions/expression.h"
#include "assignmentStatement.h"

using namespace ast;

AssignmentStatement::AssignmentStatement(Expression *target,
                                         Expression *expr,
                                         bool isBecomeStatement)
    : Statement(target->getLocation()),
      target(target),
      expr(expr),
      isBecomeStatement(isBecomeStatement)
{
    ASSERT(target);
    ASSERT(expr);
}

AssignmentStatement::~AssignmentStatement()
{
    if(!isBecomeStatement)
        delete target;
    delete expr;
}

void AssignmentStatement::contextAnalysis(DeclarationCollection &declarationCollection)
{
    target->contextAnalysis(declarationCollection);
    expr->contextAnalysis(declarationCollection);

    if(target->getType() != expr->getType())
        throw ContextAnalysisException(getLocation(), "incombatible types in assigment");

    if(!target->isLValue())
        throw ContextAnalysisException(getLocation(), "assignment target is not a l-value");

    if(!expr->isRValue())
        throw ContextAnalysisException(getLocation(), "expression is not readable");
}

void AssignmentStatement::execute(Stack2 &stack)
{
    target->execute(stack);
    expr->execute(stack);

    void *exprPtr = stack.popPtr(),
         *targetPtr = stack.popPtr();

    ASSERT(target->getType()->getDeclarationType() == Declaration::Class);
    ((ClassDeclaration*)target->getType())->assign(targetPtr, exprPtr);

    expr->executeCleanUp(stack);
    target->executeCleanUp(stack);
}

