/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../../stack.h"
#include "../declarationCollection.h"
#include "../contextAnalysisException.h"
#include "../expressions/identifierAccessExpression.h"
#include "../declarations/functionDeclaration.h"
#include "../declarations/variableDeclaration.h"
#include "../identifier.h"

#include "returnStatement.h"

using namespace ast;

ReturnStatement::ReturnStatement(const Location &location, Expression *expr)
    : AssignmentStatement(new IdentifierAccessExpression(new ResolvableIdentifier("_return_", location)),
                          expr)
{}

void ReturnStatement::contextAnalysis(DeclarationCollection &declarationCollection)
{
    if(!declarationCollection.functionDeclaration || declarationCollection.functionDeclaration->getDeclarationType() != Declaration::Function)
        throw ContextAnalysisException(getLocation(), "return statement in non function declaration context");

    FunctionDeclaration *funcDecl = (FunctionDeclaration*)declarationCollection.functionDeclaration;
    if(!funcDecl->returnVarDecl)
        throw ContextAnalysisException(getLocation(), "return statement in void function");

    AssignmentStatement::contextAnalysis(declarationCollection);

    if(funcDecl->returnVarDecl->typeIdent->declaration != expr->getType())
        throw ContextAnalysisException(getLocation(), "return type mismatch");
}

void ReturnStatement::execute(Stack2 &stack)
{
    AssignmentStatement::execute(stack);
    stack.interupt = Stack2::Return;
}

BreakStatement::BreakStatement(const Location &loc)
    : Statement(location),
      location(loc)
{}

void BreakStatement::contextAnalysis(DeclarationCollection &declarationCollection)
{
    if(!declarationCollection.forStatementStack.size())
        throw ContextAnalysisException(getLocation(), "break statement outside loop-statement");
}

void BreakStatement::execute(Stack2 &stack)
{
    stack.interupt = Stack2::Break;
}

ContinueStatement::ContinueStatement(const Location &loc)
    : Statement(location),
      location(loc)
{}

void ContinueStatement::contextAnalysis(DeclarationCollection &declarationCollection)
{
    if(!declarationCollection.forStatementStack.size())
        throw ContextAnalysisException(getLocation(), "continue statement outside loop-statement");
}

void ContinueStatement::execute(Stack2 &stack)
{
    stack.interupt = Stack2::Continue;
}

