/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../../tools/asserts.h"
#include "../../stack.h"
#include "../declarationCollection.h"
#include "../declarations/declaration.h"
#include "../declarations/variableDeclaration.h"
#include "../declarations/taskDeclaration.h"
#include "statementBlock.h"

using namespace ast;

StatementBlock::StatementList::~StatementList()
{
    for(iterator i = begin(); i != end(); i++)
        delete (*i);
    clear();
}

StatementBlock::StatementBlock()
    : Statement(Location::builtInLocation),
      internalState(0)
{}

StatementBlock::~StatementBlock()
{
    for(StatementList::iterator i = statements.begin(); i != statements.end(); i++)
        delete (*i);
    statements.clear();
    for(StatementList::iterator i = variableDeclarations.begin(); i != variableDeclarations.end(); i++)
        delete (*i);
    variableDeclarations.clear();
    for(StatementList::iterator i = functionDeclarations.begin(); i != functionDeclarations.end(); i++)
        delete (*i);
    functionDeclarations.clear();
}

void StatementBlock::contextAnalysisFunctionDeclarations(DeclarationCollection &declarationCollection)
{
    for(StatementList::iterator i = functionDeclarations.begin(); i != functionDeclarations.end(); i++)
        (*i)->contextAnalysis(declarationCollection);
}

void StatementBlock::contextAnalysisStatements(DeclarationCollection &declarationCollection)
{
    std::list<StatementList::iterator> toVariables;
    for(StatementList::iterator i = statements.begin(); i != statements.end(); i++)
    {
        if((*i)->getStatementType() == Statement::DeclarationStatement)
        {
            ASSERT(((Declaration*)*i)->getDeclarationType() == Declaration::Variable);
            toVariables.push_back(i);
        }
        (*i)->contextAnalysis(declarationCollection);
    }

    for(std::list<StatementList::iterator>::iterator i = toVariables.begin(); i != toVariables.end(); i++)
    {
        variableDeclarations.push_back(**i);
        VariableDeclaration *varDecl = (VariableDeclaration*)**i;
        if(varDecl->initializeStatement)
        {
            Statement *s = varDecl->initializeStatement;
            varDecl->initializeStatement = 0;
            **i = s;
        }
        else
            statements.erase(*i);
    }

    declarationCollection.varIdInFrame -= variableDeclarations.size();
}

void StatementBlock::contextAnalysis(DeclarationCollection &declarationCollection)
{
    declarationCollection.enter();
    contextAnalysisFunctionDeclarations(declarationCollection);
    contextAnalysisStatements(declarationCollection);
    declarationCollection.leave();
}

void StatementBlock::execute(Stack2 &stack)
{
    if(stack.owner && stack.owner->state == TaskDeclarationBase::restart)
        internalState = 0;

    switch(internalState)
    {
        case 0:
        for(StatementList::iterator i = variableDeclarations.begin(); i != variableDeclarations.end(); i++)
            (*i)->execute(stack);

        internalState = 1;
        actualStatement = statements.begin();
        case 1:

        for(; actualStatement != statements.end() && !stack.interupt;)
        {
            (*actualStatement)->execute(stack);

            if(stack.interupt == Stack2::Yield)
            {
                if((*actualStatement)->getStatementType() == Statement::Yield)
                    actualStatement++;
                return;
            }

            actualStatement++;
        }

        for(StatementList::reverse_iterator i = variableDeclarations.rbegin(); i != variableDeclarations.rend(); i++)
            (*i)->executeCleanUp(stack);
    }
    internalState = 0;
}

bool StatementBlock::isReturnStatement()
{
    for(StatementList::iterator i = statements.begin(); i != statements.end(); i++)
        if((*i)->isReturnStatement())
            return true;
    return false;
}

