/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "../../tools/asserts.h"
#include "../declarationCollection.h"
#include "../contextAnalysisException.h"
#include "../declarations/declaration.h"
#include "../declarations/taskDeclaration.h"
#include "../expressions/expression.h"
#include "yieldStatement.h"

using namespace ast;

YieldStatement::~YieldStatement()
{
    if(stateExpr)
        delete stateExpr;
}

void YieldStatement::contextAnalysis(DeclarationCollection &declarationCollection)
{
    ASSERT(declarationCollection.functionDeclaration);
    if(declarationCollection.functionDeclaration->getDeclarationType() != Declaration::Task)
        throw ContextAnalysisException(getLocation(), "yield in non task context");

    if(stateExpr)
        stateExpr->contextAnalysis(declarationCollection);
}

void YieldStatement::execute(Stack2 &stack)
{
    TaskDeclarationBase::State state = TaskDeclarationBase::running;
    if(stateExpr)
    {
        stateExpr->execute(stack);
        state = *(TaskDeclarationBase::State*)stack.popPtr();
        stateExpr->executeCleanUp(stack);
    }

    stack.yieldCounter++;
    ASSERT(stack.owner);
    stack.owner->state = state;
    stack.interupt = Stack2::Yield;
}

