/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <vector>
#include <string>

#include "callTree.h"
#include "executionContext.h"
#include "ast/declarations/native/typeToName.h"
#include "tools/tr1Include.h"

#ifndef WIN32
#define LibraryHandle void*
#else
#include <Windows.h>
#define LibraryHandle HINSTANCE
#endif

void defaultErrorHandler(const char *format, ...);

class CodeGeneratorInterface;
namespace ast
{
    class MasterModuleDeclaration;
    class ModuleDeclaration;
}

class Stack2;

class BScriptEngine
{
    public:
        BScriptEngine(const std::string &rootDir = std::string("."),
                      ErrorHandler errorHandler = &defaultErrorHandler);
        ~BScriptEngine();

        bool init();
        bool setMainCallCode(const std::string &mainCallCode);
        bool execute();
        std::string getModuleName(const std::string &filename);

        bool addModule(ast::ModuleDeclaration *module);
        bool loadModule(const std::string &filename);
        bool loadModuleLibrary(const std::string &filename, bool global = true);
        bool requires(const std::string &moduleName);

        CallTreeNode getCallTree() { return executionContext.callTree.root; };
        void clearCallTree() { executionContext.callTree.clear(); };
        bool generateCode(CodeGeneratorInterface &cg);

        template<typename T>
        bool setInput(T *t)
        {
            const TypeToName::TypeToNameMap::mapped_type *typeInfo = TypeToName::getTypeInfo<T>();
            if(!typeInfo)
            {
                executionContext.errorHandler("while setting input, class is not registered!\n");
                return false;
            }
            return setInput(typeInfo->first, typeInfo->second, t);
        }

        template<typename T>
        bool setOutput(T *t)
        {
            const TypeToName::TypeToNameMap::mapped_type *typeInfo = TypeToName::getTypeInfo<T>();
            if(!typeInfo)
            {
                executionContext.errorHandler("while setting output, class is not registered!\n");
                return false;
            }
            return setOutput(typeInfo->first, typeInfo->second, t);
        }

    private:
        ExecutionContext executionContext;
        Stack2 *mainStack;
        ast::MasterModuleDeclaration *masterModule;
        typedef std::tr1::unordered_map<std::string, bool> ModuleMap;
        ModuleMap moduleMap;
        std::string rootDir;
        std::vector<LibraryHandle> externModuleHandles;

        bool setInput(const std::string &moduleName,
                      const std::string &typeName,
                      void *ptr);
        bool setOutput(const std::string &moduleName,
                       const std::string &typeName,
                       void *ptr);
};

