/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>

#ifdef WIN32
#include "../tools/win32snprintf.h"
#endif

#include "../ast/declarations/taskDeclaration.h"
#include "../executionContext.h"
#include "../ast/declarations/native/funcWrappers.h"
#include "../ast/declarations/native/classWrappers.h"
#include "../ast/declarations/native/enumWrappers.h"

int toInt(float d)
{
    return int(d);
}

float toFloat(int i)
{
    return float(i);
}

void traces(const std::string &msg, ExecutionContext *_context)
{
    TRACE_CALL_TREE(_context->callTree, msg);
}

void tracef(const std::string &msg, float f, ExecutionContext *_context)
{
    char buf[64];
    snprintf(buf, 64, "%s: %f", msg.c_str(), f);
    TRACE_CALL_TREE(_context->callTree, buf);
}

void tracei(const std::string &msg, int i, ExecutionContext *_context)
{
    char buf[64];
    snprintf(buf, 64, "%s %d", msg.c_str(), i);
    TRACE_CALL_TREE(_context->callTree, buf);
}

void bindBuiltins(ast::ExternModuleDeclaration *module)
{
    registerClass<int>(module, "int");
    registerClass<float>(module, "float");
    registerClass<bool>(module, "bool");
    registerClass<std::string>(module, "string");

    registerEnum<ast::TaskDeclarationBase::State>(module, "TaskState")
        .value("failure", ast::TaskDeclarationBase::failure, "ast::TaskDeclarationBase::failure")
        .value("done", ast::TaskDeclarationBase::done, "ast::TaskDeclarationBase::done")
        .value("restart", ast::TaskDeclarationBase::restart, "ast::TaskDeclarationBase::restart")
        .value("running", ast::TaskDeclarationBase::running, "ast::TaskDeclarationBase::running")
        ;

    registerFunction(module, "toInt", &toInt, "int");
    registerFunction(module, "toFloat", &toFloat, "float");

    registerFullBScriptFunctionVoid(module, "TRACES", &traces, "traces");
    registerFullBScriptFunctionVoid(module, "TRACEF", &tracef, "tracef");
    registerFullBScriptFunctionVoid(module, "TRACEI", &tracei, "tracei");
}

