/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#define ENABLE_CALL_TREE
#ifdef ENABLE_CALL_TREE
    #define PUSH_CALL_TREE(ct, name) ct.pushCall(name)
    #define POP_CALL_TREE(ct) ct.popCall()
    #define TRACE_CALL_TREE(ct, msg) ct.trace(msg)
#else
    #define PUSH_CALL_TREE(ct, name) ((void)0)
    #define POP_CALL_TREE(ct) ((void)0)
    #define TRACE_CALL_TREE(ct, msg) ((void)0)
#endif

#include <vector> 
#include <string>

#include "tools/asserts.h"

struct CallTreeNode
{
    enum Type
    {
        root,
        call,
        trace,
    };

    CallTreeNode()
        : name("CallTreeNode-Without-Name!!!")
    {}

    CallTreeNode(const std::string &name, Type type)
        : name(name),
          type(type)
    {}

    std::vector<CallTreeNode> childs;
    std::string name;
    Type type;
};

class CallTreeHandler
{
    public:
        void pushCall(const std::string &name)
        {
            if(stack.size() == 0)
                stack.push_back(&root);

            CallTreeNode *p = stack.back();
            p->childs.push_back(CallTreeNode(name, CallTreeNode::call));
            stack.push_back(&p->childs.back());
        }

        void popCall()
        {
            ASSERT(stack.size() > 0);
            stack.pop_back();
        }

        void trace(const std::string &msg)
        {
            ASSERT(stack.size() > 0);
            stack.back()->childs.push_back(CallTreeNode(msg, CallTreeNode::trace));
        }

        unsigned stackSize()
        {
            return stack.size();
        }

    private:
        CallTreeNode root;
        std::vector<CallTreeNode*> stack;

        void clear()
        {
            ASSERT(stack.size() <= 1);
            root.childs.clear();
        }

        friend class BScriptEngine;
};

