/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

namespace ast
{
    class StatementBlock;
    class CallStatement;
    class IfStatement;
    class ForStatement;
    class AssignmentStatement;
    class YieldStatement;
    class ReturnStatement;
    class BreakStatement;
    class ContinueStatement;

    template <typename T>
    class LiteralExpression;
    class CallExpression;
    class IdentifierAccessExpression;
    class MemberAccessExpression;
    class UnaryExpression;
    class BinaryExpression;
    class TaskStateExpression;

    class VariableDeclaration;
    class GlobalVariableDeclaration;
    class FunctionDeclaration;
    class TaskDeclaration;
    class ClassDeclaration;
    class ModuleDeclaration;
    class ExternModuleDeclaration;
    class MasterModuleDeclaration;
    class EnumDeclaration;
}

class CodeGeneratorInterface
{
    public:
        virtual void generateCode(ast::StatementBlock &statementBlock) = 0;
        virtual void generateCode(ast::CallStatement &callStmt) = 0;
        virtual void generateCode(ast::IfStatement &ifStatement) = 0;
        virtual void generateCode(ast::ForStatement &forStatement) = 0;
        virtual void generateCode(ast::AssignmentStatement &assigStmt) = 0;
        virtual void generateCode(ast::YieldStatement &yieldStmt) = 0;
        virtual void generateCode(ast::ReturnStatement &returnStmt) = 0;
        virtual void generateCode(ast::BreakStatement &breakStmt) = 0;
        virtual void generateCode(ast::ContinueStatement &breakStmt) = 0;

        virtual void generateCode(ast::LiteralExpression<bool> &litBoolExpr) = 0;
        virtual void generateCode(ast::LiteralExpression<int> &litIntExpr) = 0;
        virtual void generateCode(ast::LiteralExpression<float> &litDoubleExpr) = 0;
        virtual void generateCode(ast::LiteralExpression<std::string> &litStringExpr) = 0;
        virtual void generateCode(ast::CallExpression &callExpr) = 0;
        virtual void generateCode(ast::IdentifierAccessExpression &varAccess) = 0;
        virtual void generateCode(ast::MemberAccessExpression &memberAccess) = 0;
        virtual void generateCode(ast::UnaryExpression &unaryExpr) = 0;
        virtual void generateCode(ast::BinaryExpression &binaryExpr) = 0;
        virtual void generateCode(ast::TaskStateExpression &taskStateExpr) = 0;

        virtual void generateCode(ast::VariableDeclaration &varDecl) = 0;
        virtual void generateCode(ast::GlobalVariableDeclaration &varDecl) = 0;
        virtual void generateCode(ast::FunctionDeclaration &funcDecl) = 0;
        virtual void generateCode(ast::TaskDeclaration &funcDecl) = 0;
        virtual void generateCode(ast::ClassDeclaration &classDecl) = 0;
        virtual void generateCode(ast::ModuleDeclaration &funcDecl) = 0;
        virtual void generateCode(ast::ExternModuleDeclaration &funcDecl) = 0;
        virtual void generateCode(ast::MasterModuleDeclaration &funcDecl) = 0;
        virtual void generateCode(ast::EnumDeclaration &funcDecl) = 0;
};

