/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <string>
#include <vector>

#include "../tools/treePrinter.h"

#include "codeGeneratorInterface.h"

namespace ast
{
class Statement;
}

class CppCodeGenerator : public CodeGeneratorInterface
{
    private:
        bool isStatement;
        bool isLocalFunctionDeclaration;
        bool isTask;
        int continueLabelID;
        int localFunctionID;
        int identifierID;
        TreePrinter *printer,
                    *headerPrinter,
                    *localPrinter;
        std::string path;
        std::vector<ast::Statement*> localFunctions;
        std::vector<int> continueLabelIDStack;

    public:
        CppCodeGenerator(const std::string &path = std::string("./"))
            : printer(0),
              headerPrinter(0),
              localPrinter(0),
              path(path)
        {}

        void generateCode(ast::StatementBlock &statementBlock);
        void generateCode(ast::CallStatement &callStmt);
        void generateCode(ast::IfStatement &ifStmt);
        void generateCode(ast::ForStatement &forStmt);
        void generateCode(ast::AssignmentStatement &assigStmt);
        void generateCode(ast::YieldStatement &yieldStmt);
        void generateCode(ast::ReturnStatement &returnStmt);
        void generateCode(ast::BreakStatement &breakStmt);
        void generateCode(ast::ContinueStatement &breakStmt);
        void generateCode(ast::LiteralExpression<bool> &litBoolExpr);
        void generateCode(ast::LiteralExpression<int> &litIntExpr);
        void generateCode(ast::LiteralExpression<float> &litFloatExpr);
        void generateCode(ast::LiteralExpression<std::string> &litStringExpr);
        void generateCode(ast::CallExpression &callExpr);
        void generateCode(ast::IdentifierAccessExpression &identifierAccess);
        void generateCode(ast::MemberAccessExpression &memAccess);
        void generateCode(ast::UnaryExpression &unaryExpr);
        void generateCode(ast::BinaryExpression &binaryExpr);
        void generateCode(ast::TaskStateExpression &taskStateExpr);
        void generateCode(ast::VariableDeclaration &varDecl);
        void generateCode(ast::GlobalVariableDeclaration &varDecl);
        void generateCode(ast::FunctionDeclaration &funcDecl);
        void generateCode(ast::TaskDeclaration &taskDecl);
        void generateCode(ast::ClassDeclaration &classDecl);
        void generateCode(ast::ModuleDeclaration &modDecl);
        void generateCode(ast::ExternModuleDeclaration &modDecl);
        void generateCode(ast::MasterModuleDeclaration &modDecl);
        void generateCode(ast::EnumDeclaration &enumDecl);
};

