/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "../tools/treePrinter.h"

#include "pythonCodeGenerator.h"
#include "../ast/ast.h"

class LuaCodeGenerator : public PythonCodeGenerator
{
    public:
        LuaCodeGenerator(const std::string &filename = std::string("-"))
            : PythonCodeGenerator(filename)
        {}

        void generateCode(ast::IfStatement &ifStmt)
        {
            printer.print("if(");
            ifStmt.expression->generateCode(*this);
            printer.print(") then");
            printer.newline();
            printer.indent();
            ifStmt.ifStatements->generateCode(*this);
            printer.dindent();

            if(ifStmt.elseStatements)
            {
                printer.println("else");
                printer.indent();
                ifStmt.elseStatements->generateCode(*this);
                printer.dindent();
            }
            printer.println("end");
        }

        void generateCode(ast::LiteralExpression<bool> &litBoolExpr)
        {
            printer.print("%s", litBoolExpr.value ? "true" : "false");
        }

        void generateCode(ast::FunctionDeclaration &funcDecl)
        {
            printer.print("function ");
            printer.print("%s", funcDecl.ident->name.c_str());
            printer.print("(");
            for(size_t i = 0; i < funcDecl.paramDecls->size(); i++)
            {
                printer.print("%s", funcDecl.paramDecls->at(i)->ident->name.c_str());
                if(i != funcDecl.paramDecls->size() - 1)
                    printer.print(", ");
            }
            printer.print(")");
            printer.newline();
            printer.indent();
            funcDecl.statements->generateCode(*this);
            printer.dindent();
            printer.println("end");
        }
};

