/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "../tools/treePrinter.h"

#include "codeGeneratorInterface.h"
#include "../ast/ast.h"

class PythonCodeGenerator : public CodeGeneratorInterface
{
    public:
        PythonCodeGenerator(const std::string &filename = std::string("-"))
            : printer(filename)
        {}

        void generateCode(ast::StatementBlock &statementBlock)
        {
            for(size_t i = 0; i < statementBlock.statements.size(); i++)
                statementBlock.statements.at(i)->generateCode(*this);
        }

        void generateCode(ast::FunctionCallStatement &funcCallStmt)
        {
            funcCallStmt.funcCall->generateCode(*this);
            printer.newline();
        }

        void generateCode(ast::IfStatement &ifStmt)
        {
            printer.print("if ");
            ifStmt.expression->generateCode(*this);
            printer.print(":");
            printer.newline();
            printer.indent();
                ifStmt.ifStatements->generateCode(*this);
            printer.dindent();

            if(ifStmt.elseStatements)
            {
                printer.println("else:");
                printer.indent();
                    ifStmt.elseStatements->generateCode(*this);
                printer.dindent();
            }
        }

        void generateCode(ast::AssignmentStatement &assigStmt)
        {
            printer.print("%s", assigStmt.target->name.c_str());
            printer.print(" = ");
            assigStmt.expr->generateCode(*this);
            printer.newline();
        }

        void generateCode(ast::LiteralExpression<bool> &litBoolExpr)
        {
            printer.print("%s", litBoolExpr.value ? "True" : "False");
        }

        void generateCode(ast::LiteralExpression<int> &litIntExpr)
        {
            printer.print("%d", litIntExpr.value);
        }

        void generateCode(ast::LiteralExpression<double> &litDoubleExpr)
        {
            printer.print("%f", litDoubleExpr.value);
        }

        void generateCode(ast::FunctionCall &funcCall)
        {
            printer.print("%s", funcCall.ident->name.c_str());
            printer.print("(");
            for(size_t i = 0; i < funcCall.paramExprs->size(); i++)
            {
                funcCall.paramExprs->at(i)->generateCode(*this);
                if(i != funcCall.paramExprs->size() - 1)
                    printer.print(", ");
            }
            printer.print(")");
        }

        void generateCode(ast::VariableDeclaration &varDecl)
        {
            printer.print("%s", varDecl.ident->name.c_str());
            printer.print(" = None");
            printer.newline();
        }

        void generateCode(ast::FunctionDeclaration &funcDecl)
        {
            printer.print("def ");
            printer.print("%s", funcDecl.ident->name.c_str());
            printer.print("(");
            for(size_t i = 0; i < funcDecl.paramDecls->size(); i++)
            {
                printer.print("%s", funcDecl.paramDecls->at(i)->ident->name.c_str());
                if(i != funcDecl.paramDecls->size() - 1)
                    printer.print(", ");
            }
            printer.print("):");
            printer.newline();
            printer.indent();
            funcDecl.statements->generateCode(*this);
            printer.dindent();
        }

    protected:
        TreePrinter printer;
};

