/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

%{
    #include <math.h>
    #include "../parserInterface.h"

    typedef yy::parser::token token;

    /*
    #define DEBUG_OUT(format, args...)             \
        if(interface.lexerDebugOut)                \
            fprintf(interface.lexerDebugOut,       \
                    "%d: " format "\n",            \
                    yylloc->begin.line,            \
                    ## args);  \
                    */
    #define DEBUG_OUT(format, ...) 

    #define yyterminate()   return token::END
%}

DIGIT       [0-9]
LETTER      [a-zA-Z][a-zA-Z0-9]*

%option yylineno
%option noyywrap
%option nounput
%option never-interactive

%x INDENT_SC
%x INDENT_SC2
%x STRING_SC
%x STRING_END

%{
#define YY_USER_ACTION  yylloc->columns(yyleng);
#define YY_NO_UNISTD_H
%}

%%

%{
    yylloc->step();
%}

<INDENT_SC>{
    [ ]+            {
                        int lastIndent = interface.indentationStack.back();
                        int newIndent = yyleng;
                        if(newIndent > lastIndent)
                        {
                            DEBUG_OUT("(indent, %d)", newIndent);
                            interface.indentationStack.push_back(newIndent);
                            BEGIN(INDENT_SC2);
                            return token::INDENT;
                        }
                        else if(newIndent < lastIndent)
                        {
                            interface.indentationStack.pop_back();
                            lastIndent = interface.indentationStack.back();
                            DEBUG_OUT("(dindent, %d)", lastIndent);
                            if(newIndent < lastIndent)
                            {
                                yyless(0);
                                return token::DINDENT;
                            }
                            else if(newIndent > lastIndent)
                            {
                                DEBUG_OUT("(indent_error)");
                                return token::INDENT_ERROR;
                            }
                            else
                            {
                                BEGIN(INDENT_SC2);
                                return token::DINDENT;
                            }
                        }
                        else
                        {
                            BEGIN(INDENT_SC2);
                            yylloc->step();
                        }
                    }
    \n              {
                        yylloc->lines();
                        yylloc->step();
                    }
    .               {
                        yyless(0);
                        yylloc->columns(-1);
                        int lastIndent = interface.indentationStack.back();
                        if(lastIndent > 0)
                        {
                            interface.indentationStack.pop_back();
                            lastIndent = interface.indentationStack.back();
                            DEBUG_OUT("(dindent, %d)", lastIndent);
                            if(lastIndent == 0)
                                BEGIN(INDENT_SC2);
                            return token::DINDENT;
                        }
                        else
                        {
                            BEGIN(INDENT_SC2);
                            yylloc->step();
                        }
                    }
    <<EOF>>         {
                        if(interface.indentationStack.size() > 1)
                        {
                            interface.indentationStack.pop_back();
                            DEBUG_OUT("(dindent, %d)", interface.indentationStack.back());
                            return token::DINDENT;
                        }
                        else
                            yyterminate();
                    }
}

<INDENT_SC2>{
    \#.*\n          {
                        BEGIN(INDENT_SC);
                        yylloc->lines();
                        yylloc->step();
                    }
    .               {
                        BEGIN(0);
                        yyless(0);
                        yylloc->columns(-1);
                        yylloc->step();
                    }
}

<STRING_SC>{
    /*
    ([^\"\n]*"\\\""*)* {
                         DEBUG_OUT("(string, %s)", yytext);
                         BEGIN(STRING_END);
                     }
                    */
    [^\"\n]*        {
                        DEBUG_OUT("(string, %s)", yytext);
                        BEGIN(STRING_END);
                        yylval->sval = new std::string(yytext);
                        return token::STRING;
                    }
    [^\"\n]*\n      {
                        DEBUG_OUT("(unterminated_string)");
                        BEGIN(0);
                        yylloc->lines();
                        return token::UNTERMINATED_STRING_ERROR;
                    }
}

<STRING_END>{
    \"              { BEGIN(0); }
}

requires            {
                        DEBUG_OUT("(requires)");
                        return token::REQUIRES;
                    }
func                {
                        DEBUG_OUT("(func)");
                        return token::FUNC;
                    }
task                {
                        DEBUG_OUT("(task)");
                        return token::TASK;
                    }
yield               {
                        DEBUG_OUT("(yield)");
                        return token::YIELD;
                    }
taskState           {
                        DEBUG_OUT("(taskState)");
                        return token::TASK_STATE;
                    }
if                  {
                        DEBUG_OUT("(if)");
                        return token::IF;
                    }
else                {
                        DEBUG_OUT("(else)");
                        return token::ELSE;
                    }
for                 {
                        DEBUG_OUT("(for)");
                        return token::FOR;
                    }
true                {
                        DEBUG_OUT("(bool, true)");
                        yylval->bval = true;
                        return token::BOOL;
                    }
false               {
                        DEBUG_OUT("(bool, false)");
                        yylval->bval = false;
                        return token::BOOL;
                    }
and                 {
                        DEBUG_OUT("(and)");
                        return token::AND;
                    }
or                  {
                        DEBUG_OUT("(or)");
                        return token::OR;
                    }
return              {
                        DEBUG_OUT("(return)");
                        return token::RETURN;
                    }
break               {
                        DEBUG_OUT("(break)");
                        return token::BREAK;
                    }
continue            {
                        DEBUG_OUT("(continue)");
                        return token::CONTINUE;
                    }
pass                {
                        DEBUG_OUT("(pass)");
                        return token::PASS;
                    }
\(                  {
                        DEBUG_OUT("(lparent)");
                        return token::LPARENT;
                    }
\)                  {
                        DEBUG_OUT("(rparent)");
                        return token::RPARENT;
                    }
\,                  {
                        DEBUG_OUT("(comma)");
                        return token::COMMA;
                    }
\.                  {
                        DEBUG_OUT("(dot)");
                        return token::DOT;
                    }
!                   {
                        DEBUG_OUT("(not)");
                        return token::NOT;
                    }
\+                  {
                        DEBUG_OUT("(plus)");
                        return token::PLUS;
                    }
-                   {
                        DEBUG_OUT("(minus)");
                        return token::MINUS;
                    }
\*                  {
                        DEBUG_OUT("(mul)");
                        return token::MUL;
                    }
\/                  {
                        DEBUG_OUT("(div)");
                        return token::DIV;
                    }
\%                  {
                        DEBUG_OUT("(mod)");
                        return token::MOD;
                    }
\+\=                {
                        DEBUG_OUT("(becomes-plus)");
                        return token::BECOMES_PLUS;
                    }
\-\=                {
                        DEBUG_OUT("(becomes-minus)");
                        return token::BECOMES_MINUS;
                    }
\/\=                {
                        DEBUG_OUT("(becomes-div)");
                        return token::BECOMES_DIV;
                    }
\*\=                {
                        DEBUG_OUT("(becomes-mul)");
                        return token::BECOMES_MUL;
                    }
\%\=                {
                        DEBUG_OUT("(becomes-mod)");
                        return token::BECOMES_MOD;
                    }
\<                  {
                        DEBUG_OUT("(lt)");
                        return token::LT;
                    }
\<\=                {
                        DEBUG_OUT("(leq)");
                        return token::LEQ;
                    }
\>                  {
                        DEBUG_OUT("(gt)");
                        return token::GT;
                    }
\>\=                {
                        DEBUG_OUT("(geq)");
                        return token::GEQ;
                    }
\=\=                {
                        DEBUG_OUT("(eq)");
                        return token::EQ;
                    }
\!\=                {
                        DEBUG_OUT("(neq)");
                        return token::NEQ;
                    }
\&\&                {
                        DEBUG_OUT("(and)");
                        return token::AND;
                    }
\|\|                {
                        DEBUG_OUT("(or)");
                        return token::OR;
                    }
\,\n[ ]+            {
                        DEBUG_OUT("(comma)");
                        /* continue line */
                        yylloc->lines();
                        yylloc->columns(yyleng - 2);
                        return token::COMMA;
                    }
\=                  {
                        DEBUG_OUT("(assign)");
                        return token::ASSIGN;
                    }
;                   {
                        DEBUG_OUT("(semicolon)");
                        return token::SEMICOLON;
                    }
:                   {
                        DEBUG_OUT("(colon)");
                        return token::COLON;
                    }
[-]?{DIGIT}+        {
                        DEBUG_OUT("(number, %d)", atoi(yytext));
                        yylval->ival = atoi(yytext);
                        return token::NUMBER;
                    }
[-]?{DIGIT}+"."{DIGIT}+ {
                        DEBUG_OUT("(float, %f)", atof(yytext));
                        yylval->fval = float(atof(yytext));
                        return token::FLOAT;
                    }
[a-zA-Z][_a-zA-Z0-9]* {
                        DEBUG_OUT("(id, %s)", yytext);
                        yylval->sval = new std::string(yytext);
                        return token::IDENTIFIER;
                    }
\"                  { BEGIN(STRING_SC); }
\"\"                {
                        DEBUG_OUT("(string, %s)", "");
                        yylval->sval = new std::string("");
                        return token::STRING;
                    }
\\\n[ ]+            {
                        /* continue line */
                        yylloc->lines();
                        yylloc->step();
                    }
\n/\n               {
                        DEBUG_OUT("empty line");
                        yylloc->lines();
                        yylloc->step();
                    }
\n[ ]+/\n           {
                        DEBUG_OUT("(not_empty_empty_line_error)");
                        yylloc->lines();
                        return token::NOT_EMPTY_EMPTY_LINE_ERROR;
                    }
\n                  {
                        DEBUG_OUT("(new_line)");
                        BEGIN(INDENT_SC);
                        yylloc->lines();
                        return token::NEW_LINE;
                    }
[ ]?#.*/\n          {
                        /* eat up comments */
                        DEBUG_OUT("comment");
                        yylloc->step();
                    }
" "                 {
                        yylloc->step();
                    }
<<EOF>>             {
                        if(interface.indentationStack.size() != 1)
                        {
                            DEBUG_OUT("(indentation_error)");
                            interface.indentationStack.pop_back();
                            return token::INDENT_ERROR;
                        }
                        yyterminate();
                    }
.                   {
                        DEBUG_OUT("(unknown_token_error)");
                        return token::UNKNOWN_TOKEN_ERROR;
                    }

%%

bool ParserInterface::openFile(const std::string &filename)
{
    ASSERT(!buffer_handle);
#ifndef WIN32
    FILE *fp = fopen(filename.c_str(), "r");
#else
    FILE *fp;
    fopen_s(&fp, filename.c_str(), "r");
#endif
    if(!fp)
    {
        errorHandler("can't open file %s", filename.c_str());
        return false;
    }
    buffer_handle = yy_create_buffer(fp, YY_BUF_SIZE);
    yy_switch_to_buffer(buffer_handle);
    return true;
}

void ParserInterface::closeFile()
{
    ASSERT(buffer_handle);
    yy_delete_buffer(buffer_handle);
    buffer_handle = 0;
    fclose(yyin);
}

bool ParserInterface::openBuffer(const char *buffer)
{
    ASSERT(!buffer_handle);
    buffer_handle = yy_scan_string(buffer);
    return true;
}

void ParserInterface::closeBuffer()
{
    ASSERT(buffer_handle);
    yy_delete_buffer(buffer_handle);
    buffer_handle = 0;
}

