/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>

#include "../tools/asserts.h"

#include "parserInterface.h"

void ParserInterface::setLexerDebugOut(const std::string &filename)
{
    ASSERT(filename.size() > 0);
    if(filename.compare("-"))
    {
#ifndef WIN32
        lexerDebugOut = fopen(filename.c_str(), "w");
#else
        fopen_s(&lexerDebugOut, filename.c_str(), "w");
#endif
    }
    else
        lexerDebugOut = stdout;
}

ParserInterface::~ParserInterface()
{
    if(lexerDebugOut)
        fclose(lexerDebugOut);
}

bool ParserInterface::parse(ast::ModuleDeclaration *moduleDecl, const std::string &filename)
{
    if(!openFile(filename))
        return false;
    actual_filename = &filename;
    yy::parser p(*this, moduleDecl, filename);

    if(p.parse())
    {
        closeFile();
        return false;
    }

    closeFile();

    return true;
}

bool ParserInterface::parse(ast::ModuleDeclaration *moduleDecl, const char *buffer)
{
    if(!openBuffer(buffer))
        return false;
    std::string filename("buffer");
    actual_filename = &filename;
    yy::parser p(*this, moduleDecl, filename);

    if(p.parse())
        return false;

    closeBuffer();

    return true;
}

void ParserInterface::error(const yy::location& l, const std::string& m)
{
    Location loc(l, *actual_filename);
    errorHandler("%s: %s", loc.toString().c_str(), m.c_str());
}

void ParserInterface::error(const std::string& m)
{
    errorHandler("%s(%s): %s", actual_filename->c_str(), m.c_str());
}

