/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

class ParserInterface;
//declare lexer function

#ifndef WIN32
#pragma GCC diagnostic ignored "-Wparentheses"
#include "bison_flex_out/parser.hpp"
#pragma GCC diagnostic warning "-Wparentheses"
#else
#include "bison_flex_out/parser.hpp"
#endif
//TODO: as soon as g++ >= 4.5 is widely used, replace the line above with
//#pragma GCC diagnostic pop

#define YY_DECL                                 \
    yy::parser::token_type yylex(yy::parser::semantic_type *yylval,    \
                                 yy::parser::location_type *yylloc,    \
                                 ParserInterface &interface)
YY_DECL;

#include <vector>
#include <string>
#include "../executionContext.h"

namespace ast
{
    class ModuleDeclaration;
}

namespace yy
{
    class location;
}

struct yy_buffer_state;

class ParserInterface
{
    public:
        std::vector<int> indentationStack;
        ErrorHandler errorHandler;
        FILE *lexerDebugOut;
        const std::string *actual_filename;

        ParserInterface(ErrorHandler errorHandler)
            : errorHandler(errorHandler),
              lexerDebugOut(0),
              buffer_handle(0)
        {
            indentationStack.push_back(0);
        }

        ~ParserInterface();

        void setLexerDebugOut(const std::string &filename);

        //parse a file, return the ast on success, NULL otherwise
        bool parse(ast::ModuleDeclaration *moduleDecl, const std::string& filename);
        bool parse(ast::ModuleDeclaration *moduleDecl, const char *buffer);

        //error output for parser and lexer
        void error(const yy::location& l, const std::string& m);
        void error(const std::string& m);

    private:
        bool openFile(const std::string &filename);
        void closeFile();

        yy_buffer_state *buffer_handle;
        bool openBuffer(const char *buffer);
        void closeBuffer();
};

