/*
b-script
Copyright (C) 2011 Thijs Jeffry de Haas

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, see <http://www.gnu.org/licenses/>.
*/

template<typename Ret>
struct Functor<Ret (*)()> : public FunctorBase
{
    typedef Ret (*T_func)();
    T_func func;

    Functor(T_func f) : FunctorBase(0), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        Ret *r = (Ret*)stack.popPtr();
        *r = func();
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1>
struct Functor<Ret (*)(P1)> : public FunctorBase
{
    typedef Ret (*T_func)(P1);
    T_func func;

    Functor(T_func f) : FunctorBase(1), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2>
struct Functor<Ret (*)(P1, P2)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2);
    T_func func;

    Functor(T_func f) : FunctorBase(2), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2, typename P3>
struct Functor<Ret (*)(P1, P2, P3)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2, P3);
    T_func func;

    Functor(T_func f) : FunctorBase(3), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2, typename P3, typename P4>
struct Functor<Ret (*)(P1, P2, P3, P4)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2, P3, P4);
    T_func func;

    Functor(T_func f) : FunctorBase(4), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2, typename P3, typename P4, typename P5>
struct Functor<Ret (*)(P1, P2, P3, P4, P5)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2, P3, P4, P5);
    T_func func;

    Functor(T_func f) : FunctorBase(5), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
struct Functor<Ret (*)(P1, P2, P3, P4, P5, P6)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2, P3, P4, P5, P6);
    T_func func;

    Functor(T_func f) : FunctorBase(6), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
struct Functor<Ret (*)(P1, P2, P3, P4, P5, P6, P7)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2, P3, P4, P5, P6, P7);
    T_func func;

    Functor(T_func f) : FunctorBase(7), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
struct Functor<Ret (*)(P1, P2, P3, P4, P5, P6, P7, P8)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2, P3, P4, P5, P6, P7, P8);
    T_func func;

    Functor(T_func f) : FunctorBase(8), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
struct Functor<Ret (*)(P1, P2, P3, P4, P5, P6, P7, P8, P9)> : public FunctorBase
{
    typedef Ret (*T_func)(P1, P2, P3, P4, P5, P6, P7, P8, P9);
    T_func func;

    Functor(T_func f) : FunctorBase(9), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            case 9: return typeid(P9).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p9 = stack.popPtr();
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        *r = func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8), 
            Converter<P9>::toConstReference(p9));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O>
struct Functor<Ret (O::*)()> : public FunctorBase
{
    typedef Ret (O::*T_func)();
    T_func func;

    Functor(T_func f) : FunctorBase(0), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)();
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O>
struct Functor<Ret (O::*)() const> : public FunctorBase
{
    typedef Ret (O::*T_func)()const;
    T_func func;

    Functor(T_func f) : FunctorBase(0, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)();
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1>
struct Functor<Ret (O::*)(P1)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1);
    T_func func;

    Functor(T_func f) : FunctorBase(1), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1>
struct Functor<Ret (O::*)(P1) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1)const;
    T_func func;

    Functor(T_func f) : FunctorBase(1, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2>
struct Functor<Ret (O::*)(P1, P2)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2);
    T_func func;

    Functor(T_func f) : FunctorBase(2), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2>
struct Functor<Ret (O::*)(P1, P2) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2)const;
    T_func func;

    Functor(T_func f) : FunctorBase(2, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3>
struct Functor<Ret (O::*)(P1, P2, P3)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3);
    T_func func;

    Functor(T_func f) : FunctorBase(3), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3>
struct Functor<Ret (O::*)(P1, P2, P3) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3)const;
    T_func func;

    Functor(T_func f) : FunctorBase(3, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4>
struct Functor<Ret (O::*)(P1, P2, P3, P4)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4);
    T_func func;

    Functor(T_func f) : FunctorBase(4), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4>
struct Functor<Ret (O::*)(P1, P2, P3, P4) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4)const;
    T_func func;

    Functor(T_func f) : FunctorBase(4, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5);
    T_func func;

    Functor(T_func f) : FunctorBase(5), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5)const;
    T_func func;

    Functor(T_func f) : FunctorBase(5, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6);
    T_func func;

    Functor(T_func f) : FunctorBase(6), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6)const;
    T_func func;

    Functor(T_func f) : FunctorBase(6, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6, P7)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6, P7);
    T_func func;

    Functor(T_func f) : FunctorBase(7), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6, P7) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6, P7)const;
    T_func func;

    Functor(T_func f) : FunctorBase(7, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6, P7, P8)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8);
    T_func func;

    Functor(T_func f) : FunctorBase(8), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6, P7, P8) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8)const;
    T_func func;

    Functor(T_func f) : FunctorBase(8, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6, P7, P8, P9)> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8, P9);
    T_func func;

    Functor(T_func f) : FunctorBase(9), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            case 9: return typeid(P9).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p9 = stack.popPtr();
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8), 
            Converter<P9>::toConstReference(p9));
        stack.pushPtr(r);
    }
};
template<typename Ret, typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
struct Functor<Ret (O::*)(P1, P2, P3, P4, P5, P6, P7, P8, P9) const> : public FunctorBase
{
    typedef Ret (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8, P9)const;
    T_func func;

    Functor(T_func f) : FunctorBase(9, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return typeid(Ret).name(); break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            case 9: return typeid(P9).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p9 = stack.popPtr();
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        Ret *r = (Ret*)stack.popPtr();
        void *o = stack.popPtr();
        *r = (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8), 
            Converter<P9>::toConstReference(p9));
        stack.pushPtr(r);
    }
};
template<>
struct FunctorVoid<void (*)()> : public FunctorBase
{
    typedef void (*T_func)();
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(0), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        func();
    }
};
template<typename P1>
struct FunctorVoid<void (*)(P1)> : public FunctorBase
{
    typedef void (*T_func)(P1);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(1), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1));
    }
};
template<typename P1, typename P2>
struct FunctorVoid<void (*)(P1, P2)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(2), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2));
    }
};
template<typename P1, typename P2, typename P3>
struct FunctorVoid<void (*)(P1, P2, P3)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2, P3);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(3), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3));
    }
};
template<typename P1, typename P2, typename P3, typename P4>
struct FunctorVoid<void (*)(P1, P2, P3, P4)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2, P3, P4);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(4), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4));
    }
};
template<typename P1, typename P2, typename P3, typename P4, typename P5>
struct FunctorVoid<void (*)(P1, P2, P3, P4, P5)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2, P3, P4, P5);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(5), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5));
    }
};
template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
struct FunctorVoid<void (*)(P1, P2, P3, P4, P5, P6)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2, P3, P4, P5, P6);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(6), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6));
    }
};
template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
struct FunctorVoid<void (*)(P1, P2, P3, P4, P5, P6, P7)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2, P3, P4, P5, P6, P7);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(7), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7));
    }
};
template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
struct FunctorVoid<void (*)(P1, P2, P3, P4, P5, P6, P7, P8)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2, P3, P4, P5, P6, P7, P8);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(8), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8));
    }
};
template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
struct FunctorVoid<void (*)(P1, P2, P3, P4, P5, P6, P7, P8, P9)> : public FunctorBase
{
    typedef void (*T_func)(P1, P2, P3, P4, P5, P6, P7, P8, P9);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(9), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            case 9: return typeid(P9).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p9 = stack.popPtr();
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        func(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8), 
            Converter<P9>::toConstReference(p9));
    }
};
template<typename O>
struct FunctorVoid<void (O::*)()> : public FunctorBase
{
    typedef void (O::*T_func)();
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(0), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)();
    }
};
template<typename O>
struct FunctorVoid<void (O::*)() const> : public FunctorBase
{
    typedef void (O::*T_func)()const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(0, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)();
    }
};
template<typename O, typename P1>
struct FunctorVoid<void (O::*)(P1)> : public FunctorBase
{
    typedef void (O::*T_func)(P1);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(1), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1));
    }
};
template<typename O, typename P1>
struct FunctorVoid<void (O::*)(P1) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(1, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1));
    }
};
template<typename O, typename P1, typename P2>
struct FunctorVoid<void (O::*)(P1, P2)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(2), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2));
    }
};
template<typename O, typename P1, typename P2>
struct FunctorVoid<void (O::*)(P1, P2) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(2, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2));
    }
};
template<typename O, typename P1, typename P2, typename P3>
struct FunctorVoid<void (O::*)(P1, P2, P3)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(3), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3));
    }
};
template<typename O, typename P1, typename P2, typename P3>
struct FunctorVoid<void (O::*)(P1, P2, P3) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(3, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(4), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(4, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(5), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(5, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(6), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(6, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6, P7)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6, P7);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(7), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6, P7) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6, P7)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(7, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6, P7, P8)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(8), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6, P7, P8) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(8, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6, P7, P8, P9)> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8, P9);
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(9), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            case 9: return typeid(P9).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p9 = stack.popPtr();
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8), 
            Converter<P9>::toConstReference(p9));
    }
};
template<typename O, typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
struct FunctorVoid<void (O::*)(P1, P2, P3, P4, P5, P6, P7, P8, P9) const> : public FunctorBase
{
    typedef void (O::*T_func)(P1, P2, P3, P4, P5, P6, P7, P8, P9)const;
    T_func func;

    FunctorVoid(T_func f) : FunctorBase(9, true), func(f) {}
    const char* getTypeIdName(int i)
    {
        switch(i)
        {
            case 0: return 0; break;
            case 1: return typeid(P1).name(); break;
            case 2: return typeid(P2).name(); break;
            case 3: return typeid(P3).name(); break;
            case 4: return typeid(P4).name(); break;
            case 5: return typeid(P5).name(); break;
            case 6: return typeid(P6).name(); break;
            case 7: return typeid(P7).name(); break;
            case 8: return typeid(P8).name(); break;
            case 9: return typeid(P9).name(); break;
            default: ASSERT(false); return 0;
        }

    }
    void call(FunctorStackInterface &stack)
    {
        void *p9 = stack.popPtr();
        void *p8 = stack.popPtr();
        void *p7 = stack.popPtr();
        void *p6 = stack.popPtr();
        void *p5 = stack.popPtr();
        void *p4 = stack.popPtr();
        void *p3 = stack.popPtr();
        void *p2 = stack.popPtr();
        void *p1 = stack.popPtr();
        void *o = stack.popPtr();
        (Converter<O>::toReference(o).*func)(Converter<P1>::toConstReference(p1), 
            Converter<P2>::toConstReference(p2), 
            Converter<P3>::toConstReference(p3), 
            Converter<P4>::toConstReference(p4), 
            Converter<P5>::toConstReference(p5), 
            Converter<P6>::toConstReference(p6), 
            Converter<P7>::toConstReference(p7), 
            Converter<P8>::toConstReference(p8), 
            Converter<P9>::toConstReference(p9));
    }
};
