requires "math.bs"
requires "io.bs"

func generateFunctor(bool returnIsNotVoid,
                     bool isMemberFunction,
                     int numOfParameters,
                     bool isConst)

    io::TreePrinter printer

    #print:
    #   template<[typename Ret, ] [typename O,] [typename P{1,....},]>
    printer.prints("template<")

    bool comma = false

    if(returnIsNotVoid)
        printer.prints("typename Ret")
        comma = true

    if(isMemberFunction)
        if(comma)
            printer.prints(", ")
        printer.prints("typename O")
        comma = true

    for(int i = 0; i < numOfParameters; i+=1)
        if(comma)
            printer.prints(", ")
        printer.prints("typename P")
        printer.printi(i+1)
        comma = true

    printer.prints(">")
    printer.newline()
    comma = false

    #print:
    #   class Functor<[Ret|void] ([O::]*)([P{1...}])> : public FunctorBase
    if(returnIsNotVoid)
        printer.prints("struct Functor<")
    else
        printer.prints("struct FunctorVoid<")

    if(returnIsNotVoid)
        printer.prints("Ret ")
    else
        printer.prints("void ")
    printer.prints("(")

    if(isMemberFunction)
        printer.prints("O::")

    printer.prints("*)(")

    for(int i = 0; i < numOfParameters; i+=1)
        if(i != 0)
            printer.prints(", ")
        printer.prints("P")
        printer.printi(i+1)

    printer.prints(")")

    if(isConst)
        printer.prints(" const")

    printer.prints("> : public FunctorBase")
    printer.newline()

    printer.println("{")
    printer.indent()

    #printer.println("public:")
    #printer.indent()

    #typedef [Ret|void] ([O::]*T_func)([P{1...}])
    printer.prints("typedef ")

    if(returnIsNotVoid)
        printer.prints("Ret ")
    else
        printer.prints("void ")

    printer.prints("(")
    if(isMemberFunction)
        printer.prints("O::")

    printer.prints("*T_func)(")

    for(int i = 0; i < numOfParameters; i+=1)
        if(i != 0)
            printer.prints(", ")
        printer.prints("P")
        printer.printi(i+1)

    printer.prints(")")
    if(isConst)
        printer.prints("const")

    printer.prints(";")

    printer.newline()

    #T_func func;
    printer.println("T_func func;")
    printer.newline()

    #ctor:
    #   Functor(T_func f) : FunctorBase(<numOfParams>, <isConst>), func(f)
    if(returnIsNotVoid)
        printer.prints("Functor(T_func f) : FunctorBase(")
    else
        printer.prints("FunctorVoid(T_func f) : FunctorBase(")
    printer.printi(numOfParameters)
    if(isConst)
        printer.prints(", true")
    printer.prints("), func(f) {}")
    printer.newline()

    #const char *getTypeIdName(int i)
    #....
    printer.println("const char* getTypeIdName(int i)")
    printer.println("{")
    printer.indent()

    printer.println("switch(i)")
    printer.println("{")
    printer.indent()
    printer.prints("case 0: return ")
    if(returnIsNotVoid)
        printer.prints("typeid(Ret).name()")
    else
        printer.prints("0")
    printer.prints("; break;")
    printer.newline()

    for(int i = 0; i < numOfParameters; i+=1)
        printer.prints("case ")
        printer.printi(i+1)
        printer.prints(": return typeid(P")
        printer.printi(i+1)
        printer.prints(").name(); break;")
        printer.newline()

    printer.prints("default: ASSERT(false); return 0;")

    printer.dindent()
    printer.println("}")
    printer.newline()

    printer.dindent()
    printer.println("}")


    #void call(FunctorStackInterface &stack)
    #....
    printer.println("void call(FunctorStackInterface &stack)")
    printer.println("{")
    printer.indent()

    for(int i = 0; i < numOfParameters; i+=1)
        printer.prints("void *p")
        printer.printi(numOfParameters - i)
        printer.prints(" = stack.popPtr();")
        printer.newline()

    if(returnIsNotVoid)
        printer.println("Ret *r = (Ret*)stack.popPtr();")

    if(isMemberFunction)
        printer.println("void *o = stack.popPtr();")

    if(returnIsNotVoid)
        printer.prints("*r = ")

    if(isMemberFunction)
        printer.prints("(Converter<O>::toReference(o).*func)(")
    else
        printer.prints("func(")

    printer.indent()
    for(int i = 0; i < numOfParameters; i+=1)
        if(i != 0)
            printer.prints(", ")
            printer.newline()
        printer.prints("Converter<P")
        printer.printi(i+1)
        printer.prints(">::toConstReference(p")
        printer.printi(i+1)
        printer.prints(")")

    printer.prints(");")
    printer.dindent()
    printer.newline()

    if(returnIsNotVoid)
        printer.prints("stack.pushPtr(r);")

    printer.dindent()
    printer.println("}")

    #printer.dindent()
    printer.dindent()
    printer.println("};")


func main()
    for(int returnsVoid = 0; returnsVoid <= 1; returnsVoid += 1)
        for(int isMember = 0; isMember <= 1; isMember += 1)
            for(int numOfParams = 0; numOfParams < 10; numOfParams += 1)
                for(int isConst = 0; isConst <= 1; isConst += 1)
                    if(isConst == 1 and isMember == 0)
                        break

                    generateFunctor(returnsVoid == 0,
                                    isMember == 1,
                                    numOfParams,
                                    isConst == 1)

