#include <stdlib.h>
#include <stdio.h>

#include "tools/asserts.h"
#include "modules2/bs_Test.h"
#include "bScriptEngine.h"
#include "codeGenerator/debugTreeGenerator.h"
#include "codeGenerator/cppCodeGenerator.h"

void usage(const char *name)
{
    printf("usage: %s <main-module(-filename)> <entry-function> { <modulefilename.so> }\n", name);
}

std::string replaceWinSlashes(const std::string &str)
{
    std::string out;
    out = str;

    while(out.find('\\') != out.npos)
    {
        size_t pos = out.find('\\');
        out.replace(pos, 1, "/");
    }

    return out;
}

int main(int argc, char **argv)
{
    if(argc <= 2)
    {
        usage(argv[0]);
        return EXIT_FAILURE;
    }

    //create engine
    BScriptEngine engine;

    //load builtins
    if(!engine.init())
        return EXIT_FAILURE;

    //load module libraries
    for(int i = 3; i < argc; i++)
    {
        if(!engine.loadModuleLibrary(argv[i]))
        {
            printf("error while loading %s\n", argv[i]);
            return EXIT_FAILURE;
        }
    }

    //set input and output
    //InputTest t;
    //t.f = 32;
    //InputTest t2;
    //if(!engine.requires("Test"))
    //    return EXIT_FAILURE;
    //if(!engine.setInput(&t))
    //    return EXIT_FAILURE;
    //if(!engine.setOutput(&t2))
    //    return EXIT_FAILURE;

    //init main call
    //mainCallCode = requires "<moduleFileName>"
    //               func main()
    //                   <moduleName>.<funcName>()
    std::string mainModulefilename = argv[1];
#ifdef WIN32
    mainModulefilename = replaceWinSlashes(mainModulefilename);
#endif
    std::string mainCallCode;
    std::string moduleName = engine.getModuleName(mainModulefilename);
    mainCallCode += "requires \"";
    mainCallCode += mainModulefilename;
    mainCallCode += "\"\n"
                    "func main()\n"
                    "   " + moduleName + "::" + argv[2] + "()\n";

    if(!engine.setMainCallCode(mainCallCode))
        return EXIT_FAILURE;
#ifdef DEBUG
    {
        DebugTreeGenerator debugGen;
        engine.generateCode(debugGen);
    }
#endif
    //{
    //    CppCodeGenerator cppCodeGen("cppOut");
    //    engine.generateCode(cppCodeGen);
    //}

    VERIFY(engine.execute());

    return EXIT_SUCCESS;
}

